from django.forms.utils import flatatt
from django.template import defaultfilters

from django_cradmin import renderable


class BreadcrumbItem(renderable.AbstractBemRenderable):
    """
    Breadcrumb item renderable.
    """
    template_name = 'django_cradmin/crbreadcrumb/breadcrumb-item.django.html'

    def __init__(self, label, url=None, active=False,
                 label_maxlength=None, active_label_maxlength=None,
                 render_link_for_active=False,
                 parent_bem_block=None):
        """

        Args:
            label (str): The text/label.
            url (str): The link URL. If this is `None` or any other boolean false value,
                we render the item as a `<span>` instead of as a `<a>` element.
            active (bool): If this is ``True``, we add the ``--active`` BEM variant to the element.
            label_maxlength (int): The max length of the label before it is shortened using the
                ``truncatechars`` django template filter.
                You typically do not set this directly, but instead override
                :meth:`.BreadcrumbItemList.get_default_item_label_maxlength` to get
                uniformed max lengths for all your breadcrumbs.
            active_label_maxlength (int): The max length of the label if active=True before the label
                is shortened using the ``truncatechars`` django template filter.
                You typically do not set this directly, but instead override
                :meth:`.BreadcrumbItemList.get_default_active_item_label_maxlength` to get
                uniformed max lengths for all your breadcrumbs.
            render_link_for_active (bool): If this is ``True``, we render as a link (``<a>``)
                even if ``active`` is ``True``. We do, of course, not render
                as a link if we do not have an ``url``.
            parent_bem_block: Provided automatically by :class:`.BreadcrumbItemList`.
        """
        self.url = url
        self.label = label
        self.active = active
        self.label_maxlength = label_maxlength
        self.active_label_maxlength = active_label_maxlength
        self.render_link_for_active = render_link_for_active
        self.parent_bem_block = parent_bem_block
        super(BreadcrumbItem, self).__init__()

    def get_bem_element(self):
        return '{}__item'.format(self.parent_bem_block)

    def get_bem_variant_list(self):
        variant_list = []
        if self.active:
            variant_list.append('active')
        return variant_list

    def should_render_as_link(self):
        """
        Should we render the element as a link (``<a>``)?
        """
        if self.active and not self.render_link_for_active:
            return False
        return bool(self.url)

    @property
    def html_tag(self):
        if self.should_render_as_link():
            return 'a'
        return 'span'

    @property
    def truncated_label(self):
        if self.active:
            label_maxlength = self.active_label_maxlength
        else:
            label_maxlength = self.label_maxlength

        if label_maxlength:
            return defaultfilters.truncatechars(self.label, label_maxlength)
        return self.label

    def get_html_element_attributes(self):
        """
        Get HTML element attributes as a dict.
        """
        html_element_attributes = {
            'class': self.css_classes or False,  # Fall back to false to avoid class=""
        }
        if self.should_render_as_link():
            html_element_attributes['href'] = self.url
        return html_element_attributes

    @property
    def html_element_attributes_string(self):
        return flatatt(self.get_html_element_attributes())


class BreadcrumbSeparator(renderable.AbstractBemRenderable):
    """
    Breadcrumb separator renderable.
    """
    template_name = 'django_cradmin/crbreadcrumb/breadcrumb-separator.django.html'

    def __init__(self, parent_bem_block=None):
        self.parent_bem_block = parent_bem_block
        super(BreadcrumbSeparator, self).__init__()

    def get_bem_element(self):
        return '{}__separator'.format(self.parent_bem_block)


class BreadcrumbItemList(renderable.AbstractBemRenderable):
    """
    List of breadcrumb items.

    We provide a lot of helper methods, but if you need more powerful features, such
    as removal of breadcrumb items, you can just manipulate the
    :attr:`~.BreadcrumbItemList.breadcrumb_item_list` attribute directly. All the append*, insert*, etc.
    methods just add items to this list.

    Attributes:
        breadcrumb_item_list (list): A plain python list of
            :class:`django_cradmin.renderable.AbstractRenderable` objects,
            normally :class:`.BreadcrumbItem` objects.
        cradmin_instance (django_cradmin.crinstance.BaseCrAdminInstance): The
            cradmin instance sent in as an argument to ``__init__()``.
    """
    template_name = 'django_cradmin/crbreadcrumb/breadcrumb-item-list.django.html'

    #: Render location: Above page cover.
    LOCATION_ABOVE_PAGE_COVER = 'above-page-cover'

    #: Render location: Below page cover.
    LOCATION_BELOW_PAGE_COVER = 'below-page-cover'

    @classmethod
    def from_breadcrumb_item_list(cls, breadcrumb_item_list, **kwargs):
        new_breadcrumb_item_list = cls(cradmin_instance=breadcrumb_item_list.cradmin_instance, **kwargs)
        new_breadcrumb_item_list.extend_with_item_renderables(breadcrumb_item_list.breadcrumb_item_list)
        return new_breadcrumb_item_list

    def __init__(self, cradmin_instance):
        """

        Args:
            cradmin_instance (django_cradmin.crinstance.BaseCrAdminInstance): A cradmin instance.
        """
        self.breadcrumb_item_list = []
        self.cradmin_instance = cradmin_instance
        self._overridden_location = None
        super(BreadcrumbItemList, self).__init__()

    def get_extra_css_classes_list(self):
        return [
            'hidden-in-print'
        ]

    def get_default_location(self):
        """
        The location to render the breadcrumb at by default.

        Can be overridden on a per view, per app or per cradmin instance
        basis using :meth:`.set_location`.

        Only relevant if using the
        :func:`django_cradmin.templatetags.cradmin_tags.cradmin_render_breadcrumb_item_list`
        template tag with the ``location`` argument.
        Defaults to :obj:`~.BreadcrumbItemList.LOCATION_BELOW_PAGE_COVER`.
        """
        return self.LOCATION_BELOW_PAGE_COVER

    def get_location(self):
        """
        Get the location to render the breadcrumb item list at.

        Do not override this - override :meth:`.get_default_location`
        instead.
        """
        return self._overridden_location or self.get_default_location()

    def set_location(self, location):
        """
        Set the location to render the breadcrumb item list at.

        Can be used in the ``get_breadcrumb_item_list_renderable()``
        method of cradmin instances, apps or views to override the
        location to render breadcrumbs at.

        Args:
            location (str): Location.
        """
        self._overridden_location = location

    def should_render_at_location(self, location):
        return location is None or location == self.get_location()

    def get_bem_block(self):
        """
        Override this to use a custom BEM block for the breadcrumb.
        """
        return 'breadcrumb-item-list'

    def get_bem_variant_list(self):
        return ['location-{}'.format(self.get_location())]

    def __len__(self):
        return len(self.breadcrumb_item_list)

    def __bool__(self):
        return len(self) > 0

    def get_separator_renderable_class(self):
        """
        Get the renderable class for breadcrumb item separators.

        Defaults to :class:`.BreadcrumbSeparator`.
        """
        return BreadcrumbSeparator

    def make_separator_renderable(self):
        """
        Make a breadcrumb item separator renderable.
        """
        return self.get_separator_renderable_class()(parent_bem_block=self.get_bem_block())

    def get_item_renderable_class(self):
        """
        Get the renderable class for breadcrumb items.

        Defaults to :class:`.BreadcrumbItem`.
        """
        return BreadcrumbItem

    def get_default_item_label_maxlength(self):
        """
        The default max length of the label of items.

        If the label is longer than this, the label is truncated
        using the ``truncatechars`` django template filter.

        If you return None or another boolean false falue from this method,
        labels are not truncated.

        Returns:
            int: The max length of item labels, or ``None``. Defaults to ``15``.
        """
        return 15

    def get_default_active_item_label_maxlength(self):
        """
        The default max length of the label of active items.

        If the label is longer than this, the label is truncated
        using the ``truncatechars`` django template filter.

        If you return None or another boolean false falue from this method,
        labels are not truncated.

        Returns:
            int: The max length of item labels, or ``None``. Defaults to ``25``.
        """
        return 25

    def get_item_renderable_kwargs(self, **extra_kwargs):
        """
        Get kwargs for the :meth:`.get_item_renderable_class`.

        Make sure you call super to get the required kwargs if you
        override this method.

        Args:
            **extra_kwargs: Extra kwargs.

        Returns:
            dict: kwargs.
        """
        kwargs = {
            'parent_bem_block': self.get_bem_block(),
            'label_maxlength': self.get_default_item_label_maxlength(),
            'active_label_maxlength': self.get_default_active_item_label_maxlength(),
        }
        kwargs.update(extra_kwargs)
        return kwargs

    def make_item_renderable(self, **kwargs):
        """
        Make an item renderable.

        Args:
            **kwargs: Kwargs for the item renderable class. This is forwarded to
                :meth:`.get_item_renderable_kwargs` to enable default kwargs for all
                items in the breadcrumb item list.
        """
        return self.get_item_renderable_class()(**self.get_item_renderable_kwargs(**kwargs))

    def append(self, **kwargs):
        """
        Uses :meth:`.make_item_renderable` to create a renderable,
        and appends the created item to the breadcrumb list.
        """
        self.append_item_renderable(self.make_item_renderable(**kwargs))

    def prepend(self, **kwargs):
        """
        Uses :meth:`.make_item_renderable` to create a renderable,
        and prepends the created item to the breadcrumb list.
        """
        self.prepend_item_renderable(self.make_item_renderable(**kwargs))

    def insert(self, index, **kwargs):
        """
        Uses :meth:`.make_item_renderable` to create a renderable,
        and inserts the created item in the breadcrumb list.
        """
        self.insert_item_renderable(index, self.make_item_renderable(**kwargs))

    def clear(self):
        self.breadcrumb_item_list.clear()

    def remove_last(self):
        if len(self.breadcrumb_item_list) > 0:
            self.breadcrumb_item_list.pop()

    def remove_first(self):
        if len(self.breadcrumb_item_list) > 0:
            del self.breadcrumb_item_list[0]

    def insert_item_renderable(self, index, renderable_object):
        """
        Insert a renderable object at a specific index in the breadcrumb list.

        Args:
            index (int): The index to insert at.
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        if index < len(self.breadcrumb_item_list):
            self.breadcrumb_item_list.insert(index, renderable_object)
        else:
            self.append_item_renderable(renderable_object)

    def prepend_item_renderable(self, renderable_object):
        """
        Prepend a renderable object to the breadcrumb list.

        Args:
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        self.breadcrumb_item_list.insert(0, renderable_object)

    def append_item_renderable(self, renderable_object):
        """
        Append a renderable object to the breadcrumb list.

        Args:
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        self.breadcrumb_item_list.append(renderable_object)

    def extend_with_item_renderables(self, renerable_iterable):
        """
        Just like :meth:`.append` except that it takes an iterable
        of renderables instead of a single renderable.
        """
        self.breadcrumb_item_list.extend(renerable_iterable)

    def iter_breadcrumb_list_renderables(self):
        """
        Iterate through all the breadcrumb items and separators.

        Separators is yielded automatically after each item except the last
        one. Separator renderables can be overridden with
        :meth:`.make_separator_renderable`.
        """
        for index, breadcrumb_item in enumerate(self.breadcrumb_item_list):
            yield breadcrumb_item
            if index < len(self.breadcrumb_item_list) - 1:
                yield self.make_separator_renderable()


class BreadcrumbItemListWrapper(renderable.AbstractBemRenderable):
    """
    Wraps a :class:`.BreadcrumbItemList` in a box.
    """
    template_name = 'django_cradmin/crbreadcrumb/breadcrumb-item-list-wrapper.django.html'

    def __init__(self, breadcrumb_item_list):
        self.breadcrumb_item_list = breadcrumb_item_list
        super(BreadcrumbItemListWrapper, self).__init__()

    def get_bem_block(self):
        return 'breadcrumb-item-list-wrapper'

    def get_bem_variant_list(self):
        return ['location-{}'.format(self.breadcrumb_item_list.get_location())]

    def get_breadcrumb_item_list_extra_context_data(self):
        return {
            'is_within_wrapper': True
        }

    @property
    def container_css_classes(self):
        return 'container container--wide'


class WrappedBreadcrumbItemList(BreadcrumbItemList):
    """
    Renders and works just like :class:`.BreadcrumbItemList` except that
    it is wrapped within a :class:`.BreadcrumbItemListWrapper`.

    You can subclass :class:`.BreadcrumbItemListWrapper` and replace the wrapper
    class by overriding :meth:`.get_wrapper_renderable_class`.
    """
    def get_wrapper_renderable_class(self):
        """
        Get the renderable for the wrapper element(s).

        Returns:
            django_cradmin.crbreadcrumb.BreadcrumbItemListWrapper: :class:`.BreadcrumbItemListWrapper`
            or a subclass.

        """
        return BreadcrumbItemListWrapper

    def get_wrapper_renderable_kwargs(self):
        """
        Get kwargs for the class returned by :meth:`.get_wrapper_renderable_class`.

        Returns:
            dict: Kwargs.
        """
        return {
            'breadcrumb_item_list': self
        }

    def make_wrapper_renderable(self):
        return self.get_wrapper_renderable_class()(**self.get_wrapper_renderable_kwargs())

    def render(self, request=None, extra_context_data=None):
        extra_context_data = extra_context_data or {}
        if extra_context_data.get('is_within_wrapper'):
            return super(WrappedBreadcrumbItemList, self).render(request=request, extra_context_data=extra_context_data)
        else:
            return self.make_wrapper_renderable().render(request=request, extra_context_data=extra_context_data)



from . import container
from . import convenience


class UnsupportedMessageLevel(Exception):
    """
    Raised by :class:`.AbstractMessageContainerMixin` when an unsupported
    message level is requested.
    """


class AbstractMessageContainerMixin(object):
    """
    Defines the interface for message lists.

    This is an abstract mixin. To use it, you inherit this mixin and
    :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`
    (or a subclass of it), and override :meth:`.create_message_container`.
    """
    #: Supported message levels for :meth:`~.AbstractMessageContainerMixin.add_message`.
    supported_message_levels = {'warning', 'success', 'info'}

    @property
    def should_render(self):
        return self.get_childcount() > 0

    def validate_message_level(self, level):
        """
        Validate that the provided ``level`` is in :obj:`.supported_message_levels`.

        Raises:
            .UnsupportedMessageLevel: If ``level`` is not in :obj:`.supported_message_levels`.
        """
        if level not in self.supported_message_levels:
            raise UnsupportedMessageLevel('{module}.{classname} does not support message level: {level}'.format(
                module=self.__class__.__module__,
                classname=self.__class__.__name__,
                level=level))

    def create_message_container(self, level, text='', **kwargs):
        """
        Create a message container object.

        The returned object must be a subclass of
        :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.

        Args:
            level: One of the message levels in :obj:`.supported_message_levels`.
            text: Message text. This is optional. The only use-case for not including
                the message text is to create a more complex message with
                child containers. In that case, you should add child containers
                via ``kwargs``.
            **kwargs: Kwargs for the message container. This means that you
                can create really complex messages by adding any
                AbstractContainerRenderable subclass as a child of the message container.

        Raise:
            .UnsupportedMessageLevel: If ``level`` is not in :obj:`.supported_message_levels` -
                you must implement this behavior in your subclass by calling
                :meth:`.validate_message_level`.

        Returns:
            The created AbstractContainerRenderable object.
        """
        raise NotImplementedError()

    def add_message(self, **kwargs):
        """
        Add a message.

        Creates the message using :meth:`.create_message_container`,
        and adds the message as a child of this container.

        Args:
            **kwargs:: Kwargs for :meth:`.create_message_container`.
        """
        message_container = self.create_message_container(**kwargs)
        return self.add_child(message_container)

    def add_warning(self, **kwargs):
        """
        Add warning message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'warning'
        return self.add_message(**kwargs)

    def add_success(self, **kwargs):
        """
        Add success message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'success'
        return self.add_message(**kwargs)

    def add_info(self, **kwargs):
        """
        Add info message.

        Args:
            **kwargs: Same as for :meth:`.add_message`, the only
                difference is that the ``level`` kwarg is set.
        """
        kwargs['level'] = 'info'
        return self.add_message(**kwargs)

    def add_validationerror(self, validationerror, prefix=None, **kwargs):
        """
        Add a :class:`django.forms.ValidationError`.

        Args:
            validationerror: A :class:`django.forms.ValidationError`
            **kwargs: Same as for :meth:`.add_warning`.
        """
        for message in validationerror.messages:
            if prefix:
                message = '{}: {}'.format(prefix, message)
            self.add_warning(text=message, **kwargs)
        return self

    def add_validationerror_list(self, validationerror_list, **kwargs):
        """
        Add a list of :class:`django.forms.ValidationError` objects.

        Just a shortcut for looping through the list and adding errors
        using :meth:`.add_validationerror`.

        Args:
            validationerror_list: A list of :class:`django.forms.ValidationError` objects.
            **kwargs: Same as for :meth:`.add_validationerror`.
        """
        for validationerror in validationerror_list:
            self.add_validationerror(validationerror=validationerror, **kwargs)
        return self


class MessageContainer(convenience.AbstractWithOptionalEscapedText):
    """
    Single message container.
    """
    level_to_variant_map = {
        'warning': 'error',
        'info': 'info',
        'success': 'info',
    }

    def __init__(self, level, **kwargs):
        """
        Args:
            level: The message level. See :meth:`.AbstractMessageContainerMixin.create_message_container`.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.level = level
        super(MessageContainer, self).__init__(**kwargs)

    def get_default_html_tag(self):
        return 'p'

    def get_default_bem_block_or_element(self):
        return 'message'

    def get_default_bem_variant_list(self):
        """
        Default BEM variant set to the level kwarg for :meth:`.__init__`.

        So this assumes that a variant matching the level exists.
        """
        level_variant = self.level_to_variant_map.get(self.level)
        if level_variant:
            return [level_variant]
        else:
            return []

    def get_default_test_css_class_suffixes_list(self):
        return super(MessageContainer, self).get_default_test_css_class_suffixes_list() + [
            '{}-message'.format(self.level or 'default')
        ]


class CompactMessageContainer(MessageContainer):
    """
    Single message container - compact version.
    """
    def get_default_bem_variant_list(self):
        variants = super(CompactMessageContainer, self).get_default_bem_variant_list()
        variants.append('compact')
        return variants


class BoxMessageContainer(convenience.AbstractWithOptionalEscapedText):
    """
    Message container using the ``box`` css class.
    """
    level_to_variant_map = {
        'error': 'warning',
        'warning': 'warning',
        'info': 'info',
        'success': 'success',
    }

    def __init__(self, level, **kwargs):
        """
        Args:
            level: The message level. See :meth:`.AbstractMessageContainerMixin.create_message_container`.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self.level = level
        super(BoxMessageContainer, self).__init__(**kwargs)

    def get_default_html_tag(self):
        return 'div'

    def get_default_bem_block_or_element(self):
        return 'box'

    def get_default_bem_variant_list(self):
        """
        Default BEM variant set to the level kwarg for :meth:`.__init__`.

        So this assumes that a variant matching the level exists.
        """
        level_variant = self.level_to_variant_map.get(self.level)
        if level_variant:
            return [level_variant]
        else:
            return []

    def get_default_test_css_class_suffixes_list(self):
        return super(BoxMessageContainer, self).get_default_test_css_class_suffixes_list() + [
            '{}-message'.format(self.level or 'default')
        ]


class PlainMessagesContainer(AbstractMessageContainerMixin,
                             container.AbstractContainerRenderable):
    """
    Messages container.
    """
    def get_message_container_class(self, level):
        return MessageContainer

    def create_message_container(self, level, text='', **kwargs):
        message_container_class = self.get_message_container_class(level=level)
        return message_container_class(level=level, text=text, **kwargs)


class MessagesContainer(PlainMessagesContainer):
    """
    Messages container - contains zero or more :class:`.MessageContainer`.
    """
    def get_default_bem_block_or_element(self):
        return 'messages'


class BoxMessagesContainer(PlainMessagesContainer):
    """
    Messages container - contains zero or more :class:`.BoxMessageContainer`.
    """
    def get_message_container_class(self, level):
        return BoxMessageContainer

    def get_default_bem_block_or_element(self):
        return 'messages'


class CompactMessagesContainer(PlainMessagesContainer):
    """
    Same as :class:`.MessagesContainer` except that it uses
    :class:`.CompactMessageContainer` instead of :class:`.MessageContainer`
    for the messages, and that it has no css class for the wrapper element
    by default.
    """
    def get_message_container_class(self, level):
        return CompactMessageContainer

    def get_default_bem_block_or_element(self):
        return None


class AdminUiPageSectionMessagesContainer(PlainMessagesContainer):
    template_name = 'django_cradmin/uicontainer/messagescontainer/adminui_page_section_messages.django.html'



from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView

from django_cradmin import javascriptregistry


class RoleSelectView(javascriptregistry.viewmixin.WithinRoleViewMixin, ListView):
    """
    The default view for listing and selecting roles within a cradmin instance.

    - If the user has a single role, we redirect to the role-frontpage for that role.
      This behavior can be overridden with :meth:`.get_autoredirect_if_single_role`.
    - If the user has multiple roles, we list the roles.
    - If the user has no roles, we call :meth:`.get_no_roles_response`.
    """
    paginate_by = 30

    #: Makes the roles queryset available as ``roles`` in the template.
    context_object_name = 'roles'

    #: The template used to render this view.
    template_name = 'django_cradmin/roleselect.django.html'

    #: The title of the page. See :meth:`.get_pagetitle`.
    pagetitle = _('What would you like to edit?')

    #: Redirect if we have a single role? See :meth:`.get_autoredirect_if_single_role`.
    autoredirect_if_single_role = True

    def get_queryset(self):
        return self.request.cradmin_instance.get_rolequeryset()

    def get(self, *args, **kwargs):
        rolecount = self.get_queryset().count()
        if rolecount == 0:
            return self.get_no_roles_response(*args, **kwargs)
        elif rolecount == 1:
            return self.get_single_role_response(*args, **kwargs)
        else:
            return self.get_multiple_roles_response(*args, **kwargs)

    def get_autoredirect_if_single_role(self):
        """
        Enables/disables automatic redirect if single role.

        Returns the value of :obj:`.autoredirect_if_single_role` by default.

        Used by :meth:`.get_single_role_response`.
        """
        return self.autoredirect_if_single_role

    def get_no_roles_response(self, *args, **kwargs):
        """
        Get the response to return if the requesting user only have no roles.

        Raises :exc:`django.core.exceptions.PermissionDenied` by default.

        If you want to do something more eloborate, you can do one of the following:

        - Use a HttpResonseRedirect to redirect to some other view/url.
        - Call ``return self.get_multiple_roles_response(*args, **kwargs)``.
          The template for this view (``django_cradmin/roleselect.django.html``) has a
          ``no_roles_section`` block. You can extend this template and
          override this block to display a custom message. You must, of course,
          set this new template as the :obj:`~.RoleSelectView.template_name`.
        """
        raise PermissionDenied()

    def get_single_role_response(self, *args, **kwargs):
        """
        Get the response to return if the requesting user only have one role.

        If :meth:`.get_autoredirect_if_single_role` returns ``True``,
        we redirect to the rolefrontpage for the role. Otherwise,
        we return :meth:`.get_multiple_roles_response`.
        """
        if self.get_autoredirect_if_single_role():
            only_role = self.get_queryset().first()
            return HttpResponseRedirect(self.request.cradmin_instance.rolefrontpage_url(
                self.request.cradmin_instance.get_roleid(only_role)))
        else:
            return super(RoleSelectView, self).get(*args, **kwargs)

    def get_multiple_roles_response(self, *args, **kwargs):
        """
        Get the response to return if the requesting user only has multiple roles.

        Just calls the ``get()``-method of the superclass by default.
        """
        return super(RoleSelectView, self).get(*args, **kwargs)

    def get_pagetitle(self):
        """
        Get the page title.

        Returns the value of :obj:`.pagetitle` by default.
        """
        return self.pagetitle

    def get_context_data(self, **kwargs):
        context = super(RoleSelectView, self).get_context_data(**kwargs)
        context['pagetitle'] = self.get_pagetitle()
        context['rolecount'] = self.get_queryset().count()
        self.add_javascriptregistry_component_ids_to_context(context=context)
        return context



from django.conf import settings
from django.utils.module_loading import import_string
from django.utils.translation import pgettext_lazy

from django_cradmin import renderable


def get_default_expandable_menu_renderable(**kwargs):
    """
    Get the default expandable menu renderable.

    The one set in the :setting:`DJANGO_CRADMIN_DEFAULT_EXPANDABLE_MENU_CLASS`.

    Args:
        **kwargs: Kwargs for the expandable menu class constructor.

    Returns:
        AbstractMenuRenderable: Expandable menu renderable object or ``None``.
    """
    if not getattr(settings, 'DJANGO_CRADMIN_DEFAULT_EXPANDABLE_MENU_CLASS', None):
        return None
    menu_class = import_string(settings.DJANGO_CRADMIN_DEFAULT_EXPANDABLE_MENU_CLASS)
    return menu_class(**kwargs)


class BaseMenuLinkRenderable(renderable.AbstractBemRenderable):
    template_name = 'django_cradmin/crmenu/menuitem/link.django.html'

    def __init__(self, label, url, is_active=False,
                 bem_variant_list=None, parent_bem_block=None,
                 **kwargs):
        """
        Parameters:
            label: A label shown in the menu.
            url: The url to go to whem the user clicks the menu item.
            is_active: Should be ``True`` if the menuitem should be styled as active.
        """
        self.label = label
        self.url = url
        self.is_active = is_active
        self.parent_bem_block = parent_bem_block
        super(BaseMenuLinkRenderable, self).__init__(bem_variant_list=bem_variant_list, **kwargs)

    def get_fallback_parent_bem_block(self):
        return 'adminui-page-header'

    def get_parent_bem_block(self):
        return self.parent_bem_block or self.get_fallback_parent_bem_block()

    def get_bem_element(self):
        return '{}__navlink'.format(self.get_parent_bem_block())

    def get_active_bem_variant_list(self):
        return [
            'active'
        ]

    def get_bem_variant_list(self):
        bem_variant_list = super(BaseMenuLinkRenderable, self).get_bem_variant_list()
        if self.is_active:
            bem_variant_list.extend(self.get_active_bem_variant_list())
        return bem_variant_list


class NavLinkItemRenderable(BaseMenuLinkRenderable):
    """
    Use this to add links to the main menu.
    """


class NavLinkButtonItemRenderable(NavLinkItemRenderable):
    """
    Use this to add links styled as a button to the main menu.
    """
    def get_bem_variant_list(self):
        return ['button']


class ExpandableMenuItem(BaseMenuLinkRenderable):
    """
    Use this to add links to the main menu.
    """
    template_name = 'django_cradmin/crmenu/menuitem/expandable-menu-item.django.html'

    def get_fallback_parent_bem_block(self):
        return 'expandable-menu'

    def get_bem_element(self):
        return '{}__link'.format(self.get_parent_bem_block())

    def get_list_element_bem_element(self):
        return '{}__listitem'.format(self.get_parent_bem_block())


class ExpandableMenuSeparator(renderable.AbstractBemRenderable):
    template_name = 'django_cradmin/crmenu/menuitem/expandable-menu-separator.django.html'

    def __init__(self, parent_bem_block=None, **kwargs):
        self.parent_bem_block = parent_bem_block
        super(ExpandableMenuSeparator, self).__init__()

    def get_fallback_parent_bem_block(self):
        return 'expandable-menu'

    def get_parent_bem_block(self):
        return self.parent_bem_block or self.get_fallback_parent_bem_block()

    def get_bem_element(self):
        return '{}__separator'.format(self.get_parent_bem_block())


class MenuToggleItemItemRenderable(renderable.AbstractBemRenderable):
    """
    Use this to add an expandable menu toggle to the menu.
    """
    template_name = 'django_cradmin/crmenu/menuitem/menutoggle.django.html'

    def __init__(self, parent_bem_block=None, **kwargs):
        self.parent_bem_block = parent_bem_block
        super(MenuToggleItemItemRenderable, self).__init__()

    def get_label(self):
        return pgettext_lazy('cradmin default header menu toggle label', 'Menu')

    def get_fallback_parent_bem_block(self):
        return 'adminui-page-header'

    def get_parent_bem_block(self):
        return self.parent_bem_block or self.get_fallback_parent_bem_block()

    def get_bem_element(self):
        return '{}__navmenutoggle'.format(self.get_parent_bem_block())

    def get_label_css_class(self):
        return '{}__navmenutoggle-label'.format(self.get_parent_bem_block())

    def get_icon_css_class(self):
        return '{}__navmenutoggle-icon'.format(self.get_parent_bem_block())


class AbstractMenuRenderable(renderable.AbstractRenderableWithCss):
    """
    Base class for rendering a menu.

    To get a completely custom HTML menu, you simply set your own template
    (see :meth:`django_cradmin.renderable.AbstractRenderable.get_template_name`)
    and write your own HTML.
    """

    #: The link renderable class used by :meth:`.make_link_renderable`.
    link_renderable_class = None

    #: The separator renderable class used by :meth:`.make_separator_renderable`.
    separator_renderable_class = None

    #: The button renderable class used by :meth:`.make_button_renderable`.
    button_renderable_class = None

    def __init__(self, request, cradmin_instance=None):
        self.request = request
        self.cradmin_instance = cradmin_instance
        self.renderable_list = []
        super(AbstractMenuRenderable, self).__init__()

    def iter_renderables(self):
        """
        Get an iterator over the items in the list.
        """
        return iter(self.renderable_list)

    def get_link_renderable_class(self):
        """
        Get the link renderable class.

        The default implementation returns :obj:`~.AbstractMenuRenderable.link_renderable_class`,
        and raises ValueError if it is ``None``.

        You can override this method, or override
        :obj:`~.AbstractMenuRenderable.link_renderable_class` to change
        the link renderable class.
        """
        if not self.link_renderable_class:
            raise ValueError('No link_renderable_class defined for {}'.format(self.__class__.__name__))
        return self.link_renderable_class

    def get_separator_renderable_class(self):
        """
        Get the separator renderable class.

        The default implementation returns :obj:`~.AbstractMenuRenderable.separator_renderable_class`,
        and raises ValueError if it is ``None``.

        You can override this method, or override
        :obj:`~.AbstractMenuRenderable.separator_renderable_class` to change
        the separator renderable class.
        """
        if not self.separator_renderable_class:
            raise ValueError('No separator_renderable_class defined for {}'.format(self.__class__.__name__))
        return self.separator_renderable_class

    def get_button_renderable_class(self):
        """
        Get the button renderable class.

        The default implementation returns :obj:`~.AbstractMenuRenderable.button_renderable_class`,
        and raises ValueError if it is ``None``.

        You can override this method, or override
        :obj:`~.AbstractMenuRenderable.button_renderable_class` to change
        the button renderable class.
        """
        if not self.button_renderable_class:
            raise ValueError('No button_renderable_class defined for {}'.format(self.__class__.__name__))
        return self.button_renderable_class

    def get_child_renderable_parent_bem_block(self):
        raise NotImplementedError()

    def make_child_renderable_kwargs(self, **kwargs):
        """
        Takes the user provided kwargs for :meth:`.make_link_renderable`
        or :meth:`.make_button_renderable` and returns the final
        kwargs that should be used as argument for the constructor
        of the renderable class.

        By default, this adds the ``parent_bem_block`` kwarg
        with the value returned by :meth:`.get_child_renderable_parent_bem_block`.
        """
        full_kwargs = {
            'parent_bem_block': self.get_child_renderable_parent_bem_block()
        }
        full_kwargs.update(**kwargs)
        return full_kwargs

    def make_link_renderable(self, **kwargs):
        """
        Make a link renderable.

        Uses the renderable returned by :meth:`.get_link_renderable_class`,
        and forwards ``**kwargs`` to the constructor of that class.

        Kwargs is filtered through :meth:`.make_child_renderable_kwargs`,
        so you can override that method to add default kwargs.

        Returns:
            A link renderable object.
        """
        return self.get_link_renderable_class()(**self.make_child_renderable_kwargs(**kwargs))

    def make_separator_renderable(self, **kwargs):
        """
        Make a separator renderable.

        Uses the renderable returned by :meth:`.get_separator_renderable_class`,
        and forwards ``**kwargs`` to the constructor of that class.

        Kwargs is filtered through :meth:`.make_child_renderable_kwargs`,
        so you can override that method to add default kwargs.

        Returns:
            A separator renderable object.
        """
        return self.get_separator_renderable_class()(**self.make_child_renderable_kwargs(**kwargs))

    def make_button_renderable(self, **kwargs):
        """
        Make a button renderable.

        Uses the renderable returned by :meth:`.get_button_renderable_class`,
        and forwards ``**kwargs`` to the constructor of that class.

        Kwargs is filtered through :meth:`.make_child_renderable_kwargs`,
        so you can override that method to add default kwargs.

        Returns:
            A button renderable object.
        """
        return self.get_button_renderable_class()(**self.make_child_renderable_kwargs(**kwargs))

    def append_link(self, **kwargs):
        """
        Uses :meth:`.make_link_renderable` to create a renderable,
        and appends the created link to the menu.
        """
        self.append(self.make_link_renderable(**kwargs))

    def prepend_link(self, **kwargs):
        """
        Uses :meth:`.make_link_renderable` to create a renderable,
        and prepends the created link to the menu.
        """
        self.prepend(self.make_link_renderable(**kwargs))

    def insert_link(self, index, **kwargs):
        """
        Uses :meth:`.make_link_renderable` to create a renderable,
        and inserts the created link in the menu.
        """
        self.insert(index, self.make_link_renderable(**kwargs))

    def append_separator(self, **kwargs):
        """
        Uses :meth:`.make_separator_renderable` to create a renderable,
        and appends the created separator to the menu.
        """
        self.append(self.make_separator_renderable(**kwargs))

    def prepend_separator(self, **kwargs):
        """
        Uses :meth:`.make_separator_renderable` to create a renderable,
        and prepends the created separator to the menu.
        """
        self.prepend(self.make_separator_renderable(**kwargs))

    def insert_separator(self, index, **kwargs):
        """
        Uses :meth:`.make_separator_renderable` to create a renderable,
        and inserts the created separator in the menu.
        """
        self.insert(index, self.make_separator_renderable(**kwargs))

    def append_button(self, **kwargs):
        """
        Uses :meth:`.make_button_renderable` to create a renderable,
        and appends the created button to the menu.
        """
        self.append(self.make_button_renderable(**kwargs))

    def prepend_button(self, **kwargs):
        """
        Uses :meth:`.make_button_renderable` to create a renderable,
        and prepends the created button to the menu.
        """
        self.prepend(self.make_button_renderable(**kwargs))

    def insert_button(self, index, **kwargs):
        """
        Uses :meth:`.make_button_renderable` to create a renderable,
        and inserts the created button in the menu.
        """
        self.insert(index, self.make_button_renderable(**kwargs))

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the menu in.

        Defaults to ``"div"``.
        """
        return 'div'

    def get_wrapper_htmltag_id(self):
        """
        Get the ID of the wrapper HTML element.
        """
        raise None

    def get_bem_block_or_element(self):
        raise NotImplementedError()

    def get_base_css_classes_list(self):
        return [self.get_bem_block_or_element()]

    def has_items(self):
        """
        Returns:
            bool: ``True`` if the list has any items, and ``False`` otherwise.
        """
        return len(self.renderable_list) > 0

    def __len__(self):
        return len(self.renderable_list)

    def __bool__(self):
        return len(self) > 0

    def insert(self, index, renderable_object):
        """
        Insert a renderable object at a specific index in the menu.

        Args:
            index (int): The index to insert at.
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        if index < len(self.renderable_list):
            self.renderable_list.insert(index, renderable_object)
        else:
            self.append(renderable_object)

    def prepend(self, renderable_object):
        """
        Prepend a renderable object to the menu.

        Args:
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        self.renderable_list.insert(0, renderable_object)

    def append(self, renderable_object):
        """
        Append a renderable object to the menu.

        Args:
            renderable_object: The renderable object (a subclass
                of :class:`django_cradmin.renderable.AbstractRenderable`)
        """
        self.renderable_list.append(renderable_object)

    def extend(self, renerable_iterable):
        """
        Just like :meth:`.append` except that it takes an iterable
        of renderables instead of a single renderable.
        """
        self.renderable_list.extend(renerable_iterable)


class DefaultMainMenuRenderable(AbstractMenuRenderable):
    """
    The default large screen (desktop) :class:`.Menu` renderable.
    """
    template_name = 'django_cradmin/crmenu/menu/default-main.django.html'
    menutoggle_renderable_class = MenuToggleItemItemRenderable
    link_renderable_class = NavLinkItemRenderable
    button_renderable_class = NavLinkButtonItemRenderable

    def get_wrapper_htmltag_id(self):
        return 'id_django_cradmin_menu_main'

    def get_menutoggle_renderable(self):
        return self.menutoggle_renderable_class(**self.make_child_renderable_kwargs())

    @property
    def cached_menutoggle_renderable(self):
        if not hasattr(self, '_cached_menutoggle_renderable'):
            self._cached_menutoggle_renderable = self.get_menutoggle_renderable()
        return self._cached_menutoggle_renderable

    @property
    def expandable_menu_renderable(self):
        if self.cradmin_instance:
            return getattr(self.cradmin_instance, 'expandable_menu_renderable', None)
        else:
            return get_default_expandable_menu_renderable(request=self.request)

    def has_expandable_menu_renderable(self):
        return bool(self.expandable_menu_renderable)

    def should_render_menutoggle(self):
        return self.has_expandable_menu_renderable() and bool(self.cached_menutoggle_renderable)

    def get_child_renderable_parent_bem_block(self):
        return 'adminui-page-header'

    def get_bem_block_or_element(self):
        return '{}__nav'.format(self.get_child_renderable_parent_bem_block())

    def __bool__(self):
        return super(DefaultMainMenuRenderable, self).__bool__() or self.has_expandable_menu_renderable()


class DefaultExpandableMenuRenderable(AbstractMenuRenderable):
    """
    The default small screen (mobile) :class:`.Menu` renderable.
    """
    template_name = 'django_cradmin/crmenu/menu/default-expandable.django.html'
    link_renderable_class = ExpandableMenuItem
    separator_renderable_class = ExpandableMenuSeparator
    button_renderable_class = None

    def get_wrapper_htmltag_id(self):
        return 'id_django_cradmin_menu_expandable'

    def get_bem_block_or_element(self):
        return 'expandable-menu'

    def get_child_renderable_parent_bem_block(self):
        return self.get_bem_block_or_element()



import django_cradmin
from django_cradmin import javascriptregistry


class CradminJavascript(javascriptregistry.component.AbstractJsComponent):
    @classmethod
    def get_component_id(cls):
        return 'django_cradmin_javascript'

    def get_sourceurls(self):
        return [
            self.get_static_url('django_cradmin_js/{version}/django_cradmin_all.js'.format(
                version=django_cradmin.__version__
            ))
        ]



from ievv_opensource.utils import ievvbuildstatic
from ievv_opensource.utils.lazy_static import LazyStatic
from ievv_opensource.utils.lazy_string import LazyString

import django_cradmin


class SassBuild(ievvbuildstatic.sassbuild.Plugin):
    def __init__(self,
                 sourcefile, sourcefolder='styles',
                 destinationfolder=None,
                 other_sourcefolders=None,
                 sass_include_paths=None,
                 sass_variables=None,
                 **kwargs):

        super(SassBuild, self).__init__(
            sourcefile=sourcefile,
            sourcefolder=sourcefolder,
            destinationfolder=destinationfolder,
            other_sourcefolders=other_sourcefolders,
            sass_include_paths=sass_include_paths,
            sass_variables=self.build_cradmin_sass_variables(sass_variables),
            **kwargs
        )

    def build_cradmin_sass_variables(self, extra_sass_variables):
        sass_variables = {
            'media-path': LazyString(
                "'{}'",
                LazyStatic('django_cradmin_styles/{}/media'.format(django_cradmin.__version__)))
        }
        if extra_sass_variables:
            sass_variables.update(extra_sass_variables)
        return sass_variables



from django.conf import settings
from django.utils.module_loading import import_string

from django_cradmin import renderable


def get_default_header_renderable(**kwargs):
    """
    Get the default header renderable.

    The one set in the :setting:`DJANGO_CRADMIN_DEFAULT_HEADER_CLASS`.

    Args:
        **kwargs: Kwargs for the header class constructor.

    Returns:
        AbstractHeaderRenderable: Header renderable object or ``None``.
    """
    if not getattr(settings, 'DJANGO_CRADMIN_DEFAULT_HEADER_CLASS', None):
        return None
    header_class = import_string(settings.DJANGO_CRADMIN_DEFAULT_HEADER_CLASS)
    return header_class(**kwargs)


class AbstractHeaderRenderable(renderable.AbstractRenderableWithCss):
    """
    Abstract base class for headers.

    You will typically want to use/extend :class:`.DefaultHeaderRenderable`
    instead of this unless you have some special needs.
    """
    def __init__(self, cradmin_instance=None, request=None):
        """

        Args:
            cradmin_instance (django_cradmin.crinstance.BaseCrAdminInstance): A
                cradmin instance object or ``None``.
            request: Not needed if you provide cradmin_instance.
        """
        self.cradmin_instance = cradmin_instance
        self.request = request
        if not request and cradmin_instance:
            self.request = cradmin_instance.request

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the header in.

        Defaults to ``"header"``.
        """
        return 'header'

    def get_wrapper_htmltag_id(self):
        """
        Get the ID of the wrapper html tag.
        """
        return 'id_django_cradmin_page_header'


class DefaultHeaderRenderable(AbstractHeaderRenderable):
    """
    The default header renderable class.
    """
    template_name = 'django_cradmin/crheader/default-header.django.html'

    @property
    def bem_block(self):
        return 'adminui-page-header'

    @property
    def bem_variants_list(self):
        """
        Get a list of BEM variants.

        These are added to the CSS classes prefixed with :meth:`bem_block`
        followed by ``--`` (``<bem_block>--<variant>``).
        """
        return []

    def expand_bem_variants_list(self):
        bem_block = self.bem_block
        return ['{}--{}'.format(bem_block, variant) for variant in self.bem_variants_list]

    def get_base_css_classes_list(self):
        return [self.bem_block] + self.expand_bem_variants_list()

    def get_main_menu_renderable(self):
        """
        Get the main menu renderable.

        Returns :meth:`django_cradmin.crinstance.BaseCrAdminInstance.main_menu_renderable`
        if we have a ``cradmin_instance``, otherwise this returns ``None``.

        You can safely override this, and you will typically want to
        do so if you use the :setting:`DJANGO_CRADMIN_DEFAULT_HEADER_CLASS`.
        """
        if self.cradmin_instance:
            return self.cradmin_instance.main_menu_renderable
        return None



from django.conf import settings
from django.utils.module_loading import import_string

from django_cradmin import renderable


def get_default_footer_renderable(**kwargs):
    """
    Get the default footer renderable.

    The one set in the :setting:`DJANGO_CRADMIN_DEFAULT_FOOTER_CLASS`.

    Args:
        **kwargs: Kwargs for the footer class constructor.

    Returns:
        AbstractFooter: Header renderable object or ``None``.
    """
    if not getattr(settings, 'DJANGO_CRADMIN_DEFAULT_FOOTER_CLASS', None):
        return None
    footer_class = import_string(settings.DJANGO_CRADMIN_DEFAULT_FOOTER_CLASS)
    return footer_class(**kwargs)


class AbstractFooter(renderable.AbstractRenderableWithCss):
    def __init__(self, cradmin_instance=None, request=None):
        self.cradmin_instance = cradmin_instance
        self.request = request
        if not request and cradmin_instance:
            self.request = cradmin_instance.request

    def get_wrapper_htmltag(self):
        """
        Get the HTML tag to wrap the footer in.

        Defaults to ``"footer"``.
        """
        return 'footer'



from .previewmixin import PreviewMixin  # noqa

from .formviewmixin import FormViewMixin  # noqa

from .formview import WithinRoleFormView  # noqa
from .formview import StandaloneFormView  # noqa

from .create_update_view_mixin import CreateUpdateViewMixin  # noqa

from .createview import CreateViewMixin  # noqa
from .createview import WithinRoleCreateView  # noqa

from .updateview import UpdateViewMixin  # noqa
from .updateview import WithinRoleUpdateView  # noqa
from .updateview import UpdateRoleView  # noqa
from .updateview import RedirectToCreateIfDoesNotExistMixin  # noqa

from .deleteview import DeleteViewMixin  # noqa
from .deleteview import WithinRoleDeleteView  # noqa



from django.contrib.auth import get_user_model
from django.urls import reverse
from django.test import TestCase, RequestFactory, override_settings
from django.utils import timezone
import htmls
from rest_framework.test import APIRequestFactory

from django_cradmin.apps.cradmin_resetpassword.api.change_password import ChangePasswordApi
from unittest import mock
from django_cradmin.apps.cradmin_resetpassword.views.reset import ResetPasswordView
from django_cradmin.apps.cradmin_generic_token_with_metadata.models import GenericTokenWithMetadata
from django_cradmin.tests.helpers import create_user


class TestResetPasswordView(TestCase):
    apiview_class = ChangePasswordApi

    def setUp(self):
        self.testuser = create_user('testuser')

    def make_request(self, method, viewkwargs=None, api_url='/test/', data=None, requestuser=None):
        factory = APIRequestFactory()
        request = getattr(factory, method)(api_url, format='json', data=data)
        viewkwargs = viewkwargs or {}
        if requestuser:
            request.user = requestuser
        response = self.apiview_class.as_view()(request, **viewkwargs)
        response.render()
        return response

    def make_post_request(self, viewkwargs=None, api_url='/test/', data=None, requestuser=None):
        return self.make_request(method='post', viewkwargs=viewkwargs,
                                 api_url=api_url, data=data,
                                 requestuser=requestuser)

    def test_ok_password(self):
        user = self.testuser
        user.set_password('testpassword')
        user.save()

        self.assertTrue(user.check_password('testpassword'))
        response = self.make_post_request(
            requestuser=user,
            data={
                'old_password': 'testpassword',
                'new_password': 'otherpassword'
            })
        user.refresh_from_db()
        self.assertTrue(user.check_password('otherpassword'))

    def test_wrong_password(self):
        user = self.testuser
        user.set_password('testpassword')
        user.save()

        self.assertTrue(user.check_password('testpassword'))
        response = self.make_post_request(
            requestuser=user,
            data={
                'old_password': 'wrong',
                'new_password': 'otherpassword'
            })
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.data, {'old_password': 'Invalid password'})
        user.refresh_from_db()
        self.assertTrue(user.check_password('testpassword'))

    @override_settings(
        AUTH_PASSWORD_VALIDATORS=[
            {
                'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
                'OPTIONS': {
                    'min_length': 10,
                }
            }
        ]
    )
    def test_validate(self):
        user = self.testuser
        user.set_password('testpassword')
        user.save()

        self.assertTrue(user.check_password('testpassword'))
        response = self.make_post_request(
            requestuser=user,
            data={
                'old_password': 'testpassword',
                'new_password': 'oth'
            })

        self.assertEqual(response.status_code, 400)
        self.assertEqual(str(response.data['password']),
                         'This password is too short. It must contain at least 10 characters.')


from django.conf import settings
from django.utils.module_loading import import_string
from django.views.generic import FormView

from django_cradmin import javascriptregistry


class RegisterAccountView(FormView, javascriptregistry.viewmixin.StandaloneBaseViewMixin):
    template_name = 'cradmin_register_account/register_account.django.html'

    def get_form_class(self):
        if self.form_class:
            return self.form_class
        else:
            return import_string(settings.DJANGO_CRADMIN_REGISTER_ACCOUNT_FORM_CLASS)

    def get_context_data(self, **kwargs):
        context = super(RegisterAccountView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        form = context['form']
        context['form_renderable'] = form.get_form_renderable()
        context['DJANGO_CRADMIN_SITENAME'] = settings.DJANGO_CRADMIN_SITENAME
        return context

    def get_success_url(self):
        """
        Get the next url to go to after the user has been created.

        Defaults to the ``DJANGO_CRADMIN_REGISTER_ACCOUNT_REDIRECT_URL``, falling back to
        the ``LOGIN_URL`` setting.
        """
        if 'next' in self.request.GET:
            return self.request.GET['next']
        else:
            return str(getattr(
                settings,
                'DJANGO_CRADMIN_REGISTER_ACCOUNT_REDIRECT_URL',
                settings.LOGIN_URL))

    def form_valid(self, form):
        self.created_user = form.save()
        return super(RegisterAccountView, self).form_valid(form)



from django.core.exceptions import ValidationError
from rest_framework.exceptions import ValidationError as ApiValidationError
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from django.contrib.auth.password_validation import validate_password


class ChangePasswordApi(APIView):
    permission_classes = [IsAuthenticated]

    def post(self, *args, **kwargs):
        old_password = self.request.data.get('old_password')
        new_password = self.request.data.get('new_password')
        user = self.request.user
        if not user.check_password(old_password):
            raise ApiValidationError({'old_password': 'Invalid password'})
        if not new_password:
            raise ApiValidationError({'new_password': 'Valid password must be provided'})
        try:
            validate_password(new_password)
        except ValidationError as error:
            raise ApiValidationError({'password': error.messages[0]})

        user.set_password(new_password)
        user.save()
        return Response({'message': 'New password set'})



from django.views.generic.detail import DetailView as DjangoDetailView

from django_cradmin import javascriptregistry
from django_cradmin.viewhelpers.mixins import QuerysetForRoleMixin, CommonCradminViewMixin


class DetailView(javascriptregistry.viewmixin.WithinRoleViewMixin,
                 CommonCradminViewMixin, QuerysetForRoleMixin, DjangoDetailView):
    """
    Base detail view for Django cradmin views.

    .. note:: You should import this class with ``from django_cradmin import viewhelpers``,
        and refer to it using ``viewhelpers.detail.DetailView``.
    """

    def get_context_data(self, **kwargs):
        context = super(DetailView, self).get_context_data(**kwargs)
        self.add_javascriptregistry_component_ids_to_context(context=context)
        self.add_common_view_mixin_data_to_context(context=context)
        return context


class DetailRoleView(DetailView):
    """
    Extends :class:`.DetailView` to streamline creating a detail view
    for the current role object.

    Just like :class:`.DetailView`, but with the get_object and
    get_queryset_for_role methods implemented to edit the current role
    object.

    .. note:: You should import this class with ``from django_cradmin import viewhelpers``,
        and refer to it using ``viewhelpers.detail.DetailRoleView``.
    """
    def get_object(self, queryset=None):
        return self.get_queryset_for_role().get()

    def get_queryset_for_role(self):
        return self.model.objects.filter(pk=self.request.cradmin_role.pk)



from django.urls import path

from django_cradmin.apps.cradmin_activate_account.views.activate import ActivateAccountView

urlpatterns = [
    path('activate/<str:token>',
         ActivateAccountView.as_view(),
         name="cradmin-activate-account-activate"),
]



import os
import posixpath
import uuid
from builtins import object

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.template.loader import render_to_string
from django.utils.translation import gettext_lazy as _

from django_cradmin import crsettings
from django_cradmin.utils.deprecation_decorators import CradminDeprecatedSinceV4


CradminDeprecatedSinceV4(
    message='Deprecated, and only available to enable migration of the data. '
            'If you still need cradmin_imagearchive, please use the cradmin_legacy '
            'library, which can be used along with django_cradmin>=4.0.0. '
            'See the releasenotes for 4.0.0 for more details and migration guide.'
).show_warning(name=__name__)


def archiveimage_upload_to(archiveimage, filename):
    if archiveimage.id is None:
        raise ValueError('Can not set image until after the ArchiveImage object has been created.')
    name, extension = os.path.splitext(filename)
    filenamepattern = getattr(settings, 'DJANGO_CRADMIN_IMAGEARCHIVE_FILENAMEPATTERN',
                              'cradmin_imagearchive_images/{id}-{uuid}{extension}')
    return filenamepattern.format(
        id=archiveimage.id,
        uuid=uuid.uuid1(),
        extension=extension)


class ArchiveImageManager(models.Manager):
    def filter_owned_by_role(self, role):
        return self.get_queryset().filter(
            role_object_id=role.id,
            role_content_type=ContentType.objects.get_for_model(role.__class__))


class ArchiveImageDeprecated(models.Model):
    objects = ArchiveImageManager()

    role_content_type = models.ForeignKey(
        ContentType,
        verbose_name=_('role'),
        help_text=_('The role owning this image.'),
        on_delete=models.CASCADE)
    role_object_id = models.PositiveIntegerField()
    role = GenericForeignKey('role_content_type', 'role_object_id')

    #: The image.
    image = models.FileField(
        max_length=255, null=True, blank=False,
        verbose_name=_('image'),
        help_text=_('Select an image to add to the archive.'),
        upload_to=archiveimage_upload_to)

    #: The height of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_height = models.IntegerField(null=True, blank=True, editable=False)

    #: The width of the :obj:`.image`. Autopopulated by the :obj:`.image` field.
    image_width = models.IntegerField(null=True, blank=True, editable=False)

    #: The file extension
    file_extension = models.CharField(
        max_length=255, blank=False, null=False, editable=False)

    #: The file size in bytes.
    file_size = models.PositiveIntegerField(blank=True, null=True)

    #: The name of the image file.
    #:
    #: Can be None. This is both because this was added
    #: after cradmin was in use in production, and because
    #: calculating size can be expensive if uploading
    #: huge amounts of images in bulk. All the cradmin views
    #: provided by the app sets this.
    name = models.CharField(
        max_length=255, blank=True, null=False,
        verbose_name=_('name')
        # help_text=_('A good name helps search engines find the image, '
        #             'and it helps visually impaired users.')
    )

    #: An optional description of the image.
    description = models.TextField(
        blank=True, null=False, default='',
        verbose_name=_('description'),
        help_text=_(
            'An optional description of the image. Think if this as a description '
            'of the image for visually impaired users. This means that you should describe '
            'the information carried in the image (if any). A good description also helps '
            'search engines find the image.'),
    )

    #: Create datetime.
    created_datetime = models.DateTimeField(
        editable=False,
        auto_now_add=True
    )

    class Meta(object):
        verbose_name = _('archive image')
        verbose_name_plural = _('archive images')

    def clean(self):
        if not self.name:
            if self.image:
                self.name = os.path.splitext(posixpath.basename(self.image.name))[0]
        if self.name:
            self.file_extension = os.path.splitext(self.name)[1]

    def __str__(self):
        return self.name

    def __repr__(self):
        return 'ArchiveImage(name={name}, image_width={image_width}, image_height={image_height})'.format(
            name=self.name.encode('ascii', 'replace'),
            image_width=self.image_width,
            image_height=self.image_height)

    def get_preview_html(self, request, imagetype=None):
        """
        Get HTML that can be used to show a preview of the image.

        Parameters:
            request: A HttpRequest object which is used to create an
                absolute URI for the image.
        """
        if not imagetype:
            imagetype = crsettings.get_setting('DJANGO_CRADMIN_IMAGEARCHIVE_PREVIEW_IMAGETYPE')
        context = {
            'archiveimage': self,
            'imagetype': imagetype,
            'fallbackoptions': {
                'width': 300,
                'height': 300
            }
        }
        return render_to_string('django_cradmin/apps/cradmin_imagearchive/preview.django.html',
                                context, request=request)

    @property
    def screenreader_text(self):
        """
        Get screen reader text for the image. Typically added to the "alt"
        or "aria-label" HTML attributes.

        Defaults to description, but falls back to name.
        """
        if self.description:
            return self.description
        else:
            return self.name



try:
    import django.forms.renderers  # noqa
except ImportError:
    _has_template_based_form_rendering = False
else:
    _has_template_based_form_rendering = True


def has_template_based_form_rendering():
    return _has_template_based_form_rendering



import datetime
import json
from xml.sax.saxutils import quoteattr

from django import forms

from django.utils.safestring import mark_safe
from django.utils.translation import gettext, pgettext

from django_cradmin.widgets import rangeinput
from . import container
from . import form_mixins
from . import label
from . import utils

if utils.has_template_based_form_rendering():
    from django.forms.renderers import get_default_renderer


class BaseFieldRenderable(container.AbstractContainerRenderable, form_mixins.FieldWrapperRenderableChildMixin):
    """
    Abstract base class for renders of the actual form field
    for a :class:`.FieldWrapper`.

    You never use this on its own outside a :class:`.FieldWrapper`.
    """
    def __init__(self, autofocus=False, placeholder=False, **kwargs):
        """

        Args:
            autofocus (boolean): Autofocus on this field at page load?
            placeholder (str): Placeholder text. See
                :meth:`~django_cradmin.uicontainer.container.AbstractContainerRenderable.get_html_element_attributes`
                for details about how values are applied.
            **kwargs: Kwargs for :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        self._overridden_autofocus = autofocus
        self._overridden_placeholder = placeholder
        super(BaseFieldRenderable, self).__init__(**kwargs)

    def get_default_autofocus(self):
        """
        Get the default value for the autofocus attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def fieldname(self):
        return self.field_wrapper_renderable.fieldname

    @property
    def value(self):
        return self.bound_formfield.value()

    @property
    def label_for_dom_id(self):
        return self.dom_id

    @property
    def autofocus(self):
        """
        Get the value for the autofocus attribute of the html element.

        You should not override this. Override :meth:`.get_default_autofocus` instead.
        """
        return self._overridden_autofocus or self.get_default_autofocus()

    def get_default_placeholder(self):
        """
        Get the default value for the placeholder attribute of the html element.

        Defaults to ``False``.
        """
        return False

    @property
    def placeholder(self):
        """
        Get the value for the placeholder attribute of the html element.

        You should not override this. Override :meth:`.get_default_placeholder` instead.
        """
        return self._overridden_placeholder or self.get_default_placeholder()

    def get_field_element_attributes(self):
        """
        Get the attributes of the field element.

        Useful if you create a subclass where the template wraps the field
        in another element. You can then use :meth:`.get_html_element_attributes`
        to get the attributes of the wrapper element, and override this
        to set attributes on the field. See :class:`.Select` for an example.

        Defaults to :meth:`.get_html_element_attributes` with placeholder
        and autofocus added.
        """
        field_element_attributes = dict(self.get_html_element_attributes())
        field_element_attributes.update({
            'autofocus': self.autofocus,
            'placeholder': self.placeholder,
        })
        return field_element_attributes

    def should_have_for_attribute_on_label(self):
        """
        Should this field get a ``for`` attribute on its label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.

        Defaults to ``True``, but subclasses can override this.
        """
        return True

    def should_render_as_child_of_label(self):
        """
        Should this field be renderered as a child of the ``<label>``?

        Returns ``True`` by default, but subclasses should override this
        for fields that should not be rendered within a label.

        For example, :class:`.Field` overrides this with varying behavior
        depending on the type of the Django field widget.
        """
        return True

    @property
    def bound_formfield(self):
        return self.field_wrapper_renderable.formrenderable.form[self.field_wrapper_renderable.fieldname]

    def get_default_dom_id(self):
        return self.bound_formfield.auto_id

    @property
    def field_attributes_dict(self):
        attributes_dict = {}
        attributes_dict = self.bound_formfield.build_widget_attrs(attributes_dict)
        attributes_dict.update(self.get_field_element_attributes())
        return attributes_dict


class SubWidgetField(BaseFieldRenderable):
    template_name = 'django_cradmin/uicontainer/field/subwidget.django.html'

    def __init__(self, django_subwidget, index_in_parent, **kwargs):
        self.django_subwidget = django_subwidget
        self.index_in_parent = index_in_parent
        super(SubWidgetField, self).__init__(**kwargs)

    @property
    def label_text(self):
        if utils.has_template_based_form_rendering():
            return self.django_subwidget.get('label', '')
        else:
            return self.django_subwidget.choice_label

    def can_have_children(self):
        return False

    @property
    def field_attributes_dict(self):
        attributes_dict = {}
        if utils.has_template_based_form_rendering():
            attrs = self.django_subwidget.get('attrs', {})
        else:
            attrs = self.django_subwidget.attrs
        attributes_dict.update(attrs)
        attributes_dict.update(self.get_html_element_attributes())
        if self.dom_id:
            if utils.has_template_based_form_rendering():
                attributes_dict['id'] = self.dom_id
            else:
                attributes_dict['id'] = '{dom_id}_{index_in_parent}'.format(
                    dom_id=self.dom_id,
                    index_in_parent=self.index_in_parent)
        return attributes_dict

    @property
    def rendered_subwidget(self):
        if utils.has_template_based_form_rendering():
            renderer = get_default_renderer()
            widget = self.bound_formfield.field.widget
            attrs = self.field_attributes_dict
            attrs['id'] = self.dom_id

            option = widget.create_option(
                name=self.django_subwidget['name'],
                value=self.django_subwidget.get('value', None),
                label=self.django_subwidget.get('label'),
                selected=self.django_subwidget['selected'],
                index=self.index_in_parent,
                subindex=None,
                attrs=self.field_attributes_dict
            )
            if isinstance(option, dict) and option.get('type') in ('checkbox', 'radio'):
                option['wrap_label'] = False
            return mark_safe(renderer.render(
                template_name=self.django_subwidget['template_name'],
                context={
                    'widget': option
                }))
        else:
            return self.django_subwidget.tag(attrs=self.field_attributes_dict)


class Field(BaseFieldRenderable):
    """
    Automatically renders a form field for a :class:`.FieldWrapper`
    using the Django widget system.

    You never use this on its own outside a :class:`.FieldWrapper`.

    Examples:

        First of all, this is always used within a
        :class:`~django_cradmin.uicontainer.fieldwrapper.FieldWrapper`,
        and by default FieldWrapper uses this class without any kwargs
        as the ``field_renderable``. So the following will render a
        field on the form named ``email`` no matter what the type or
        widget of the field is::

            from django_cradmin import uicontainer

            class ExampleForm(forms.Form):
                email = forms.EmailField()

            # You would put the fieldwrapper as a child somewhere below
            # a ``django_cradmin.uicontainer.form.Form``, but we
            # do not include that in these examples for brevity. See the docs for
            # ``django_cradmin.uicontainer.form.Form`` for a complete example.
            uicontainer.fieldwrapper.FieldWrapper(fieldname='email')

        Let us adjust some parameters of the field rendered by the FieldWrapper
        by providing a custom field_renderable::

            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    autofocus=True,  # Autofocus on page load
                    placeholder="Type your email...",  # Set a placeholder text

                    # Change the css classes - the css class of the field will
                    # be ``custom-input custom-input--big``
                    bem_block='custom-input',
                    bem_variant_list=['big']

                    # Alternatively, we can set the css class
                ))

        What about some CSS classes on the input field? We can do that using
        BEM naming, by specifying css classes directly as a list,
        or by just adding some css classes. Lets look at the BEM options::

            # Change the BEM block from ``input`` (the default) to ``custom-input``.
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_block='custom-input',
                    bem_variant_list=['big']
                ))

            # .. or just add some variants ..
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_variant_list=['inline', 'small']
                ))

            # .. or both ..
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='email',
                field_wrapper_renderable=uicontainer.field.Field(
                    bem_block='custom-input',
                    bem_variant_list=['large']
                ))

            # In all cases, the resulting CSS class will be
            # <bem_block> [<bem_block>--<variant0]  [<bem_block>--<variantN>]

        Refer to :ref:`uicontainer_styling` for more details about setting
        CSS classes.

        Field just works automatically with choice fields, but that uses a
        ``<select>`` by default. Let us render a ChoiceField as a
        radio button list instead::

            class ExampleForm(forms.Form):
                user_type = forms.ChoiceField(
                    choices=[
                        ('standard', 'Standard'),
                        ('admin', 'Admin'),
                    ],
                    initial='standard',
                    widget=forms.RadioSelect()
                )

            # Nothing special is needed - we pick up that you override
            # the widget on the form field, and just render it differently.
            uicontainer.fieldwrapper.FieldWrapper(fieldname='user_type')

            # We can override options for the sub-widgets (the radio buttons)
            # Lets change the CSS class of the label around each of the radio buttons
            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='user_type',
                field_renderable=uicontainer.field.Field(
                    subwidget_renderable_kwargs={
                        'bem_variant_list': ['inline']
                    }
                )
            )

        CheckboxSelectMultiple widgets work just like the RadioSelect widget.

        If you need more customization than the ``subwidget_renderable_kwargs`` option
        provides, you can customize exactly how RadioSelect and CheckboxSelectMultiple is rendered
        by extending :class:`.Field` and override:

        - :meth:`~.Field.make_subwidget_renderable_radio`
        - :meth:`~.Field.make_subwidget_renderable_checkbox`

    """
    template_name = 'django_cradmin/uicontainer/field/field.django.html'

    def __init__(self, subwidget_renderable_kwargs=None, **kwargs):
        """

        Args:
            subwidget_renderable_kwargs: Kwargs for the subwidget renderable.
                See :meth:`.make_subwidget_renderable`. You can provide any
                kwargs that works with
                :class:`~django_cradmin.uicontainer.container.AbstractContainerRenderable`.
            **kwargs: Kwargs for :class:`.BaseFieldRenderable`.
        """
        self.extra_subwidget_renderable_kwargs = subwidget_renderable_kwargs or {}
        super(Field, self).__init__(**kwargs)
        self.properties['field_wrapper_renderable'] = self

    @property
    def django_widget(self):
        """
        Get the Django Widget object for the formfield.
        """
        return self.bound_formfield.field.widget

    def is_radio_select_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.RadioSelect`.
        """
        return isinstance(self.django_widget, forms.RadioSelect)

    def is_range_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django_cradmin.widgets.rangeinput.RangeInput`.
        """
        return isinstance(self.django_widget, rangeinput.RangeInput)

    def is_checkbox_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.CheckboxInput`.
        """
        return isinstance(self.django_widget, forms.CheckboxInput)

    def is_checkbox_select_multiple_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.CheckboxSelectMultiple`.
        """
        return isinstance(self.django_widget, forms.CheckboxSelectMultiple)

    def is_select_multiple_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.SelectMultiple`.
        """
        return isinstance(self.django_widget, forms.SelectMultiple)

    def is_select_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.Select`.
        """
        return isinstance(self.django_widget, forms.Select)

    def is_checkbox_input_widget(self):
        """
        Returns ``True`` if the field widget is a :class:`django.forms.widgets.CheckboxInput`.
        """
        return isinstance(self.django_widget, forms.CheckboxInput)

    def should_render_as_subwidgets(self):
        """
        Returns True if we should render using :meth:`.make_subwidget_renderable`.

        Default to returining ``True`` if :meth:`.is_radio_select_widget` or
        :meth:`.is_checkbox_select_multiple_widget` returns ``True``.
        """
        return (self.is_radio_select_widget()
                or self.is_checkbox_select_multiple_widget())

    def should_have_for_attribute_on_label(self):
        """
        Should this field get a ``for`` attribute on its label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.

        Defaults to :meth:`.should_render_as_child_of_label`.
        """
        return not self.should_render_as_child_of_label()

    def should_render_as_child_of_label(self):
        """
        Should this field be rendered as a child of the label?

        Used by :class:`django_cradmin.uicontainer.label.Label`.
        """
        if self.is_checkbox_widget() or self.should_render_as_subwidgets():
            return True
        return False

    def get_default_bem_block_or_element(self):
        """
        Default BEM block is ``input``.
        """
        if self.is_select_widget() or self.is_checkbox_input_widget():
            return None
        elif self.is_range_widget():
            return 'range-input'
        else:
            return 'input'

    def get_default_bem_variant_list(self):
        return ['outlined']

    def get_default_test_css_class_suffixes_list(self):
        return super(Field, self).get_default_test_css_class_suffixes_list() + [
            'djangowidget-{}'.format(self.django_widget.__class__.__name__.lower())
        ]

    def render_bound_formfield_as_widget(self):
        return self.bound_formfield.as_widget(attrs=self.field_attributes_dict)

    def get_subwidget_renderable_kwargs(self, django_subwidget, index_in_parent):
        """
        Get kwargs for the subwidget created by :meth:`.make_subwidget_renderable`.

        Defaults to the ``subwidget_renderable_kwargs`` kwargs + adding
        a :class:`.SubWidgetField` object as ``subwidget_field_renderable``.
        """
        kwargs = dict(self.extra_subwidget_renderable_kwargs)
        kwargs.update({
            'subwidget_field_renderable': SubWidgetField(django_subwidget=django_subwidget,
                                                         index_in_parent=index_in_parent)
        })
        return kwargs

    def make_subwidget_renderable_radio(self, django_subwidget, index_in_parent):
        """
        Make subwidget renderable to use if :meth:`.is_radio_select_widget` returns ``True``.

        Returns a :class:`django_cradmin.uicontainer.label.RadioSubWidgetLabel`
        with kwargs from :meth:`.get_subwidget_renderable_kwargs` by default.

        Used by :meth:`.make_subwidget_renderable`.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.
        """
        return label.RadioSubWidgetLabel(**self.get_subwidget_renderable_kwargs(
            django_subwidget=django_subwidget,
            index_in_parent=index_in_parent
        ))

    def make_subwidget_renderable_checkbox(self, django_subwidget, index_in_parent):
        """
        Make subwidget renderable to use if :meth:`.is_checkbox_select_multiple_widget` returns ``True``.

        Returns a :class:`django_cradmin.uicontainer.label.CheckboxSubWidgetLabel`
        with kwargs from :meth:`.get_subwidget_renderable_kwargs` by default.

        Used by :meth:`.make_subwidget_renderable`.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.
        """
        return label.CheckboxSubWidgetLabel(**self.get_subwidget_renderable_kwargs(
            django_subwidget=django_subwidget,
            index_in_parent=index_in_parent
        ))

    def make_subwidget_renderable(self, django_subwidget, index_in_parent):
        """
        Make a renderable for a Django SubWidget.

        Only used if :meth:`.should_render_as_subwidgets` returns ``True``.

        By default this returns:

        - :meth:`.make_subwidget_renderable_radio` if :meth:`.is_radio_select_widget` returns ``True``.
        - :meth:`.make_subwidget_renderable_checkbox` if :meth:`.is_checkbox_select_multiple_widget` returns ``True``.
        - Raise NotImplementedError otherwise.

        Args:
            django_subwidget: The SubWidget object.
            index_in_parent: The index of the subwidget in the parent.

        """
        if self.is_radio_select_widget():
            return self.make_subwidget_renderable_radio(
                django_subwidget=django_subwidget, index_in_parent=index_in_parent)
        elif self.is_checkbox_select_multiple_widget():
            return self.make_subwidget_renderable_checkbox(
                django_subwidget=django_subwidget, index_in_parent=index_in_parent)
        else:
            widget_class = self.django_widget.__class__
            raise NotImplementedError(
                'The {widget_class!r} widget is not supported.'.format(
                    widget_class='{}.{}'.format(widget_class.__module__, widget_class.__name__)
                ))

    def iter_subwidgets(self):
        for index, django_subwidget in enumerate(self.django_widget.subwidgets(
                name=self.bound_formfield.html_name,
                value=self.bound_formfield.value(),
                attrs=self.field_attributes_dict)):
            subwidget = self.make_subwidget_renderable(
                django_subwidget=django_subwidget,
                index_in_parent=index)
            subwidget.bootstrap(parent=self)
            yield subwidget

    @property
    def rendered_field(self):
        return self.render_bound_formfield_as_widget()


class Select(Field):
    """
    Renders ``<select>`` wrapped in a label with
    the ``select select--outlined select--block`` css
    classes.
    """
    template_name = 'django_cradmin/uicontainer/field/select.django.html'

    def get_default_bem_block_or_element(self):
        return 'select'

    def get_default_html_tag(self):
        return 'label'

    def get_default_bem_variant_list(self):
        return ['outlined', 'block']

    def get_html_element_attributes(self):
        html_element_attributes = super(Select, self).get_html_element_attributes()
        html_element_attributes.pop('id')
        return html_element_attributes

    def get_field_element_attributes(self):
        return {
            'id': self.dom_id
        }


class HiddenField(Field):
    """
    Just like :class:`.Field`, but renders as
    a ``<input type="hidden">``.

    Typically rendered within a :class:`django_cradmin.uicontainer.fieldwrapper.NoWrapperElementFieldWrapper`.

    Examples:

        Basic example::

            uicontainer.fieldwrapper.NoWrapperElementFieldWrapper(
                fieldname='myfield',
                field_renderable=uicontainer.field.HiddenField()
            )
    """
    @property
    def rendered_field(self):
        return self.bound_formfield.as_hidden(attrs=self.field_attributes_dict)

    def get_default_bem_block_or_element(self):
        return None


class AbstractDate(Field):
    template_name = 'django_cradmin/uicontainer/field/date.django.html'

    string_datetime_formats = [
        '%Y-%m-%d',
        '%Y-%m-%d %H:%M',
        '%Y-%m-%d %H:%M:%S',
        '%H:%M',
        '%H:%M:%S',
    ]

    def __init__(self, debug=False, **kwargs):
        """

        Args:
            debug (boolean): If this is ``True``, we render the input field
                as a text input instead of a hidden input.
            **kwargs: Kwargs for :class:`.Field`.
        """
        self.debug = debug
        super(AbstractDate, self).__init__(**kwargs)

    def should_render_as_child_of_label(self):
        return False

    def get_month_labels(self):
        return [
            pgettext('monthname', 'Jan'),
            pgettext('monthname', 'Feb'),
            pgettext('monthname', 'Mar'),
            pgettext('monthname', 'Apr'),
            pgettext('monthname', 'May'),
            pgettext('monthname', 'Jun'),
            pgettext('monthname', 'Jul'),
            pgettext('monthname', 'Aug'),
            pgettext('monthname', 'Sep'),
            pgettext('monthname', 'Oct'),
            pgettext('monthname', 'Nov'),
            pgettext('monthname', 'Des')
        ]

    def make_aria_label(self, subfield_label):
        label_renderable = self.field_wrapper_renderable.label_renderable
        label_text = getattr(label_renderable, 'label_text', '')
        if label_text:
            return '{} - {}'.format(subfield_label, label_text)
        return subfield_label

    def make_day_field_props(self):
        return {
            "labelText": gettext('Day'),
            "extraSelectAttributes": {
                "aria-label": self.make_aria_label(gettext('Day'))
            }
        }

    def make_month_field_props(self):
        return {
            "labelText": gettext('Month'),
            "monthLabels": self.get_month_labels(),
            "extraSelectAttributes": {
                "aria-label": self.make_aria_label(gettext('Month'))
            }
        }

    def make_year_field_props(self):
        return {
            "labelText": gettext('Year'),
            "extraSelectAttributes": {
                "aria-label": self.make_aria_label(gettext('Year'))
            }
        }

    def make_hour_field_props(self):
        return {
            "extraInputAttributes": {
                "aria-label": self.make_aria_label(gettext('Hour'))
            }
        }

    def make_minute_field_props(self):
        return {
            "extraInputAttributes": {
                "aria-label": self.make_aria_label(gettext('Minute'))
            }
        }

    def make_result_field_props(self):
        if self.debug:
            input_type = 'text'
        else:
            input_type = 'hidden'
        return {
            "inputName": self.fieldname,
            "inputType": input_type,
        }

    def make_initial_values_dict(self, value_object):
        initial_values_dict = {}
        if isinstance(value_object, (datetime.date, datetime.datetime)):
            initial_values_dict.update({
                'initialDay': value_object.day,
                'initialMonth': value_object.month,
                'initialYear': value_object.year,
            })
        if isinstance(value_object, (datetime.time, datetime.datetime)):
            initial_values_dict.update({
                'initialHour': value_object.hour,
                'initialMinute': value_object.minute
            })
        return initial_values_dict

    def get_initial_values(self):
        value = self.value
        if value and isinstance(value, str):
            value = self.make_value_object_from_string(value_string=value)
        if value:
            return self.make_initial_values_dict(value_object=value)
        return None

    def make_widget_config_dict(self):
        config_dict = {
            "signalNameSpace": self.fieldname,
            "resultFieldProps": self.make_result_field_props(),
            "dayFieldProps": self.make_day_field_props(),
            "monthFieldProps": self.make_month_field_props(),
            "yearFieldProps": self.make_year_field_props(),
            "hourFieldProps": self.make_hour_field_props(),
            "minuteFieldProps": self.make_minute_field_props(),
        }
        initial_values = self.get_initial_values()
        if initial_values:
            config_dict.update(initial_values)
        return config_dict

    def _quoted_json(self, dct):
        return mark_safe(quoteattr(json.dumps(dct)))

    def make_value_object_from_string(self, value_string):
        for datetime_format in self.string_datetime_formats:
            try:
                return datetime.datetime.strptime(value_string,
                                                  datetime_format)
            except ValueError:
                pass
        return None

    def get_context_data(self, **kwargs):
        context = super(AbstractDate, self).get_context_data(**kwargs)
        context['widget_config_json'] = self._quoted_json(
            self.make_widget_config_dict())
        return context


class Date(AbstractDate):
    """
    Date input field renderable.

    This requires the cradmin javascript, so you will typically need
    to add::

        def get_javascriptregistry_component_ids(self):
            return ['django_cradmin_javascript']

    to your views (assuming the use one of the ``django_cradmin.viewhelpers``
    views).

    Examples:

        Basic example::

            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='birth_date',
                field_renderable=uicontainer.field.Date())
    """
    def make_widget_config_dict(self):
        widget_config_dict = super(Date, self).make_widget_config_dict()
        widget_config_dict['includeDate'] = True
        widget_config_dict['includeTime'] = False
        return widget_config_dict

    @property
    def label_for_dom_id(self):
        return '{}_daysubfield'.format(self.dom_id)

    def make_day_field_props(self):
        props = super(Date, self).make_day_field_props()
        props['extraSelectAttributes']['id'] = self.label_for_dom_id
        return props


class DateTime(AbstractDate):
    """
    DateTime input field renderable.

    This requires the cradmin javascript, so you will typically need
    to add::

        def get_javascriptregistry_component_ids(self):
            return ['django_cradmin_javascript']

    to your views (assuming the use one of the ``django_cradmin.viewhelpers``
    views).

    Examples:

        Basic example::

            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='publishing_datetime',
                field_renderable=uicontainer.field.DateTime())
    """

    @property
    def label_for_dom_id(self):
        return '{}_daysubfield'.format(self.dom_id)

    def make_day_field_props(self):
        props = super(DateTime, self).make_day_field_props()
        props['extraSelectAttributes']['id'] = self.label_for_dom_id
        return props

    def make_widget_config_dict(self):
        widget_config_dict = super(DateTime, self).make_widget_config_dict()
        widget_config_dict['includeDate'] = True
        widget_config_dict['includeTime'] = True
        return widget_config_dict


class Time(AbstractDate):
    """
    Time input field renderable.

    This requires the cradmin javascript, so you will typically need
    to add::

        def get_javascriptregistry_component_ids(self):
            return ['django_cradmin_javascript']

    to your views (assuming the use one of the ``django_cradmin.viewhelpers``
    views).

    Examples:

        Basic example::

            uicontainer.fieldwrapper.FieldWrapper(
                fieldname='current_time',
                field_renderable=uicontainer.field.Time())
    """

    @property
    def label_for_dom_id(self):
        return '{}_hoursubfield'.format(self.dom_id)

    def make_hour_field_props(self):
        props = super(Time, self).make_hour_field_props()
        props['extraInputAttributes']['id'] = self.label_for_dom_id
        return props

    def make_widget_config_dict(self):
        widget_config_dict = super(Time, self).make_widget_config_dict()
        widget_config_dict['includeDate'] = False
        widget_config_dict['includeTime'] = True
        return widget_config_dict



import json
from xml.sax.saxutils import quoteattr

from django.utils.safestring import mark_safe

from . import field
from . import container


class FilterListContainerMixin(object):
    template_name = 'django_cradmin/uicontainer/filterlist/filterlist.django.html'

    def initialize(self,
                   id_attribute='id',
                   class_name=None,
                   select_mode=None,
                   auto_load_first_page=True,
                   get_items_api_url=None,
                   components=None,
                   initially_selected_item_ids=None):
        self._id_attribute = id_attribute
        self._class_name = class_name
        self._select_mode = select_mode
        self._auto_load_first_page = auto_load_first_page
        self._get_items_api_url = get_items_api_url
        self._components = components or []
        self._initially_selected_item_ids = initially_selected_item_ids or []

    def get_config_dict(self):
        return {
            'idAttribute': self._id_attribute,
            'className': self._class_name,
            'selectMode': self._select_mode,
            'autoLoadFirstPage': self._auto_load_first_page,
            'getItemsApiUrl': self._get_items_api_url,
            'components': self._components,
            'initiallySelectedItemIds': self._initially_selected_item_ids
        }

    @property
    def widget_config_json(self):
        return mark_safe(quoteattr(json.dumps(self.get_config_dict())))


class FilterListField(FilterListContainerMixin, field.BaseFieldRenderable):
    def __init__(self,
                 id_attribute='id',
                 class_name=None,
                 select_mode=None,
                 auto_load_first_page=True,
                 get_items_api_url=None,
                 components=None,
                 initially_selected_item_ids=None,
                 **kwargs):
        self.initialize(id_attribute=id_attribute,
                        class_name=class_name,
                        select_mode=select_mode,
                        auto_load_first_page=auto_load_first_page,
                        get_items_api_url=get_items_api_url,
                        components=components,
                        initially_selected_item_ids=initially_selected_item_ids)
        super(FilterListField, self).__init__(**kwargs)

    def should_render_as_child_of_label(self):
        return False


class FilterListContainer(FilterListContainerMixin, container.AbstractContainerRenderable):
    def __init__(self,
                 id_attribute='id',
                 class_name=None,
                 select_mode=None,
                 auto_load_first_page=True,
                 get_items_api_url=None,
                 components=None,
                 initially_selected_item_ids=None,
                 **kwargs):
        self.initialize(id_attribute=id_attribute,
                        class_name=class_name,
                        select_mode=select_mode,
                        auto_load_first_page=auto_load_first_page,
                        get_items_api_url=get_items_api_url,
                        components=components,
                        initially_selected_item_ids=initially_selected_item_ids)
        super(FilterListContainer, self).__init__(**kwargs)



from django.apps import AppConfig
from . import static_components


class CradminJavascriptAppConfig(AppConfig):
    default = True
    name = 'django_cradmin.apps.django_cradmin_js'
    verbose_name = 'Django cradmin javascript'

    def ready(self):
        from django_cradmin import javascriptregistry
        javascriptregistry.Registry.get_instance().add(static_components.CradminJavascript)



from django.conf import settings
from django.contrib.auth import views as auth_views


class CradminLogoutView(auth_views.LogoutView):
    template_name = 'cradmin_authenticate/logout.django.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["LOGIN_URL"] = str(settings.LOGIN_URL)
        return context


def cradmin_logoutview(request, template_name='cradmin_authenticate/logout.django.html'):
    return CradminLogoutView.as_view(template_name=template_name)(request)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-03-07 15:23
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_temporaryfileuploadstore', '0005_auto_20180307_1622'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='TemporaryFile',
            new_name='TemporaryFileDeprecated',
        ),
    ]



from builtins import object

from django import forms
from django.conf import settings
from django.contrib.auth import get_user_model
from django.utils.translation import gettext_lazy as _

from django_cradmin import uicontainer


class AbstractCreateAccountForm(forms.ModelForm):
    """
    Base class for account creation forms.

    Subclasses **must** override:

    - :meth:`~.AbstractCreateAccountForm.set_password`.
    - :meth:`~.AbstractCreateAccountForm.get_field_renderables`.
    """

    class Meta(object):
        model = get_user_model()
        fields = []

    def get_submit_button_label(self):
        """
        Returns the submit button label.

        Override this to provide a custom label.
        """
        return _('Sign up for %(sitename)s') % {'sitename': settings.DJANGO_CRADMIN_SITENAME}

    def set_password(self, user):
        """
        Set the password of the given ``user``.

        **Must** be overridden in subclasses.

        Should not save the user, only set the password.

        Typically this will retrieve the password from the form, or
        generate a password, and set the password using ``user.set_password``. Example::

            class MyCreateAccountForm(AbstractCreateAccountForm):
                ...
                def set_password(self, user):
                    raw_password = 'secret'  # or self.cleaned_data['password']
                    user.set_password(raw_password)

        """
        raise NotImplementedError()

    def set_extra_user_attributes(self, user):
        """
        Override this to set extra user attributes in addition to
        :meth:`~.AbstractCreateAccountForm.deactivate_user` and
        :meth:`~.AbstractCreateAccountForm.set_password`.
        """

    def set_extra_user_attributes_before_clean(self, user):
        """
        Override this to set extra user attributes before the any cleaning is run on the user instance.
        """

    def get_field_renderables(self):
        """
        Get field renderables.

        Must be overridden in subclasses.

        Returns:
            list: List of :class:`django_cradmin.uicontainer.container.AbstractContainerRenderable`.
        """
        raise NotImplementedError()

    def get_submit_button_renderables(self):
        return [
            uicontainer.button.SubmitPrimary(text=self.get_submit_button_label())
        ]

    def get_form_renderable(self):
        """
        Get a :class:`django_cradmin.renderable.AbstractRenderable` that renders
        the form.

        This will typically be a :doc:`uicontainer` tree containing a
        :class:`django_cradmin.uicontainer.form.Form`, but it can be any
        AbstractRenderable. Not using a :class:`django_cradmin.uicontainer.form.Form`
        (or a subclass of it) is fairly complex when it comes to handling error
        messages and form rendering, so it is generally not recommended.

        Returns:
            django_cradmin.renderable.AbstractRenderable: The renderable object.
        """
        return uicontainer.form.Form(
            form=self,
            children=[
                uicontainer.layout.AdminuiPageSectionTight(
                    children=self.get_field_renderables() + self.get_submit_button_renderables()
                )
            ]
        ).bootstrap()

    def save(self, commit=True):
        user = super(AbstractCreateAccountForm, self).save(commit=False)
        self.set_password(user)
        self.set_extra_user_attributes(user)
        user.full_clean()
        if commit:
            user.save()
        return user

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.set_extra_user_attributes_before_clean(user=self.instance)


class AbstractCreateAccountWithPasswordForm(AbstractCreateAccountForm):
    """
    Extends :class:`.AbstractCreateAccountForm` with the typical
    password + repeat password fields. Validates that the passwords
    match.

    Subclasses **must** override:

    - :meth:`~.AbstractCreateAccountForm.deactivate_user`.
    """

    #: The first password.
    password1 = forms.CharField(
        label=_('Type your password'),
        widget=forms.PasswordInput)

    #: The repeat password fields.
    password2 = forms.CharField(
        label=_('Type your password one more time'),
        widget=forms.PasswordInput)

    def set_password(self, user):
        """
        Set the password of the user using ``user.set_password(...)``.

        Does not save the user.
        """
        raw_password = self.cleaned_data['password1']
        user.set_password(raw_password)

    def clean(self):
        """
        Validate the form.

        The default implementation validates that the passwords
        match.

        Make sure to call super if you override this::

            class MyCreateAccountForm(AbstractCreateAccountWithPasswordForm):
                ...
                def clean(self):
                    cleaned_data = super(MyCreateAccountForm, self).clean()
                    # ...
                    return cleaned_data
        """
        cleaned_data = super(AbstractCreateAccountWithPasswordForm, self).clean()
        password1 = cleaned_data.get("password1")
        password2 = cleaned_data.get("password2")

        if password1 and password2:
            if password1 != password2:
                raise forms.ValidationError(
                    message=_('The passwords do not match.'),
                    code='passwords_do_not_match')
        return cleaned_data



class QuerysetForRoleMixin(object):
    """
    Common mixin class for views that query for items limited to
    items accessible by the current cradmin role.

    .. note:: You should import this class with ``from django_cradmin import viewhelpers``,
        and refer to it using ``viewhelpers.mixins.QuerysetForRoleMixin``.
    """
    def get_queryset_for_role(self):
        """
        Get a queryset with all objects of ``self.model``  that
        the current role can access.
        """
        raise NotImplementedError()

    def get_queryset(self):
        """
        DO NOT override this. Override :meth:`.get_queryset_for_role`
        instead.
        """
        queryset = self.get_queryset_for_role()
        return queryset


class CommonCradminViewMixin(object):
    """
    Common mixin class for all cradmin view classes.

    .. note:: You should import this class with ``from django_cradmin import viewhelpers``,
        and refer to it using ``viewhelpers.mixins.CommonCradminViewMixin``.
    """
    def add_breadcrumb_list_items(self, breadcrumb_item_list):
        """
        Add items to the breadcrumb item list.

        If you completely override the :meth:`.get_breadcrumb_item_list_renderable` method
        without calling super (or calling this method explicitly), this method will have no effect.

        Examples:

            Simple example::

                def add_breadcrumb_list_items(self, breadcrumb_item_list):
                    breadcrumb_item_list.append(url='#', label='Test')

        Args:
            breadcrumb_item_list (django_cradmin.crbreadcrumb.BreadcrumbItemList): The breadcrumb item list
                to add items to.

        .. seealso:: :doc:`/crbreadcrumb`
        """

    def get_breadcrumb_item_list_renderable(self):
        """
        Get a breadcrumb item list renderable to use when rendering the template for this view.

        By default, this just uses ``request.cradmin_app.get_breadcrumb_item_list_renderable()``
        (see :meth:`django_cradmin.crapp.App.get_breadcrumb_item_list_renderable`).

        You will normally only want to override this if you want to customize how breadcrumbs
        are rendered. If you just need to add items to the breadcrumb item list, override
        :meth:`.add_breadcrumb_list_items`.

        If you override this, remember that the breadcrumb item list from
        ``request.cradmin_app.get_breadcrumb_item_list_renderable()``
        can be ``None``, so if you use that method you have to remember to handle this.

        Returns:
            django_cradmin.crbreadcrumb.BreadcrumbItemList: A breadcrumb item list renderable object
                or ``None``.

        .. seealso:: :doc:`/crbreadcrumb`
        """
        if hasattr(self.request, 'cradmin_app'):
            breadcrumb_item_list = self.request.cradmin_app.get_breadcrumb_item_list_renderable()
            if breadcrumb_item_list is not None:
                self.add_breadcrumb_list_items(breadcrumb_item_list=breadcrumb_item_list)
                return breadcrumb_item_list
        return None

    def add_common_view_mixin_data_to_context(self, context):
        """
        Call this to add common template context variables to the provided ``context``.

        Do not override this in subclasses, but if you create a class that uses this
        mixin, you **must** call this in your ``get_context_data()``-method.

        Args:
            context (dict): A template context.
        """
        context['cradmin_breadcrumb_item_list'] = self.get_breadcrumb_item_list_renderable()



import time

from django.conf import settings


class DelayMiddleware(object):
    """
    To use this, you must add the following to your settings:

    - Add ``django_cradmin.delay_middleware.DelayMiddleware`` to ``MIDDLEWARE_CLASSES``.
    - Set ``DJANGO_CRADMIN_DELAY_MIDDLEWARE_MILLISECONDS`` to the number of milliseconds
      delay you want to add to all requests (I.E.: 2000 for 2 seconds).
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        delay_seconds = getattr(settings, 'DJANGO_CRADMIN_DELAY_MIDDLEWARE_MILLISECONDS', None)
        if delay_seconds:
            time.sleep(delay_seconds / 1000.0)
        return self.get_response(request)



from django.apps import AppConfig


class WithoutStyleguideAppConfig(AppConfig):
    default = True
    name = 'django_cradmin.apps.django_cradmin_styles'
    verbose_name = 'Django cradmin styles'


class WithStyleguideAppConfig(WithoutStyleguideAppConfig):
    default = False

    def ready(self):
        from django_cradmin.apps.cradmin_kss_styleguide import styleguide_registry
        super(WithStyleguideAppConfig, self).ready()

        styleguide = styleguide_registry.CradminStyleGuide(
            unique_id='basetheme',
            label='Django CRadmin basetheme',
            appname='django_cradmin_styles',
            sourcefolder='styles/basetheme',
            sourcefile='styleguide.scss',
            # filename_patterns=[
            #     '*cradmin_theme_base/*',
            #     '*cradmin_theme_full/1__components/*',
            #     '*cradmin_theme_full/2__components/*',
            #     '*cradmin_theme_full/3__components/*',
            #     '*cradmin_theme_full/4__components/_adminui-page-header.scss',
            # ]
        )
        styleguide_registry.Registry.get_instance().add(styleguide)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-03-06 03:56
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_imagearchive', '0004_auto_20180306_0447'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ArchiveImage',
            new_name='ArchiveImageDeprecated',
        ),
    ]



from functools import wraps

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.decorators import login_required
from django.core.exceptions import ObjectDoesNotExist, PermissionDenied
from django.http import HttpResponse
from django.http import QueryDict
from django.shortcuts import resolve_url, redirect
from django.urls import reverse
from django.utils.encoding import force_str

from django_cradmin.registry import cradmin_instance_registry


def has_access_to_cradmin_instance(cradmin_instance_id, view_function,
                                   redirect_field_name=REDIRECT_FIELD_NAME,
                                   login_url=None):
    """
    Decorator for django_cradmin views.

    Makes it impossible to access the view unless the
    :meth:`django_cradmin.crinstance.BaseCrAdminInstance.has_access` method
    returns ``True``.

    Adds the following variables to the request:

        - ``cradmin_instance``: The :class:`django_cradmin.crinstance.BaseCrAdminInstance`
          registered with the provided ``cradmin_instance_id``.
    """

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        cradmin_instance = cradmin_instance_registry.get_instance_by_id(
            id=cradmin_instance_id,
            request=request)
        if cradmin_instance.has_access():
            request.cradmin_instance = cradmin_instance
            # Check if two-factor authentication is required
            if cradmin_instance.get_two_factor_auth_viewname():
                return two_factor_required(view_function)(request, *args, **kwargs)
            return view_function(request, *args, **kwargs)
        elif request.user.is_authenticated:
            raise PermissionDenied()
        else:
            # Redirect to login just like login_required()
            from django.contrib.auth.views import redirect_to_login
            path = request.build_absolute_uri()
            resolved_login_url = force_str(
                resolve_url(login_url or settings.LOGIN_URL))
            return redirect_to_login(path, resolved_login_url, redirect_field_name)

    return wrapper


def cradminview(view_function):
    """
    Decorator for django_cradmin views.

    Protects the view, making it impossible to access unless the requesting user
    has the role defined by the named url variable ``roleid``.

    Adds the following variables to the request:

        - ``cradmin_role``: The detected cradmin role. This is ``None``
            if the ``roleclass`` attribute of the cradmin instance is ``None``.
    """

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        if request.cradmin_instance.roleclass:
            roleid = kwargs.pop('roleid')
            role = request.cradmin_instance.get_role_from_roleid(roleid)
            if not role:
                return request.cradmin_instance.invalid_roleid_response(roleid)
            try:
                role_from_rolequeryset = request.cradmin_instance.get_role_from_rolequeryset(role)
            except ObjectDoesNotExist:
                response = request.cradmin_instance.missing_role_response(role)
            else:
                request.cradmin_role = role_from_rolequeryset
                response = view_function(request, *args, **kwargs)
        else:
            request.cradmin_role = None
            response = view_function(request, *args, **kwargs)

        if isinstance(response, HttpResponse):
            http_headers = request.cradmin_instance.get_common_http_headers()
            if http_headers:
                for headerattribute, headervalue in http_headers.items():
                    response[headerattribute] = headervalue

        request.cradmin_instance.add_extra_instance_variables_to_request(request)
        return response

    return wrapper


def two_factor_required(view_function, urlname=None):
    """
    Decorator for django_cradmin views.

    Adds the support for two-factor authentication.
    Will redirect the user to the implemented two-factor authentication view specified in
    settings with `DJANGO_CRADMIN_TWO_FACTOR_AUTH_URLNAME` unless `two_factor_verified` has been
    added to the session data (this must be handled in the two-factor authentication view).
    """
    urlname = urlname or getattr(settings, 'DJANGO_CRADMIN_TWO_FACTOR_AUTH_URLNAME', None)

    @wraps(view_function)
    def wrapper(request, *args, **kwargs):
        if urlname and not request.session.get('two_factor_verified'):
            url = reverse(getattr(settings, 'DJANGO_CRADMIN_TWO_FACTOR_AUTH_URLNAME', None))
            querystring = QueryDict(mutable=True)
            querystring['next'] = request.get_full_path()
            url = '{}?{}'.format(url, querystring.urlencode())
            return redirect(url)
        return view_function(request, *args, **kwargs)
    return login_required(wrapper)



from django.shortcuts import redirect

from . import generic


class UiMock(generic.StandaloneBaseTemplateView):
    """
    View for UI mocks.

    See :doc:`viewhelpers_uimock` for details.
    """

    #: Template directory (prefix for the templates of all the mocks).
    #:
    #: Must end with ``/``. The directory must contain an ``index.django.html`` template.
    #:
    #: You typically send this as an argument to ``UiMock.as_view()`` (I.E.: you
    #: normally do not need to subclass UiMock).
    #:
    #: See :meth:`.get_template_names` for details on how this is used.
    template_directory = None

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if not self.template_directory:
            raise ValueError('The UiMock class requires a template_directory as argument to as_view().')
        if not self.template_directory.endswith('/'):
            raise ValueError('The template_directory kwarg for UiMock must end with /')

    def dispatch(self, request, mockname=None, **kwargs):
        """

        Args:
            request: The HttpRequest.
            mockname: The name of the requested mock. We expect that a template named
                ``<mockname>.django.html`` is available in :obj:`~.UiMock.template_directory`.
            **kwargs: Other view kwargs (future proofing).

        Returns:

        """
        self.session_postdata = None
        self.mockname = mockname
        if 'cradmin_uimock_postdata' in self.request.session:
            self.session_postdata = self.request.session['cradmin_uimock_postdata']
            if 'csrfmiddlewaretoken' in self.session_postdata:
                del self.session_postdata['csrfmiddlewaretoken']
            del self.request.session['cradmin_uimock_postdata']

        return super().dispatch(request, mockname, **kwargs)

    def _get_mock_template_name(self, mockname=None):
        template_name = 'index.django.html'
        if mockname:
            template_name = '{}.django.html'.format(mockname)
        template_name = '{}{}'.format(self.template_directory, template_name)
        return template_name

    def get_template_names(self):
        """
        Looks up a template in the :obj:`~.UiMock.template_directory` by the following rules:

        - If we DO NOT have a ``mockname`` in view kwargs (see :meth:`.dispatch`), we
          return ``<template_directory>/index.django.html``.
        - If we have a ``mockname`` in view kwargs (see :meth:`.dispatch`), we
          return ``<template_directory>/<mockname>.django.html``.
        """
        return [
            self._get_mock_template_name(mockname=self.mockname),
            self._get_mock_template_name(mockname='404'),
            'django_cradmin/viewhelpers/uimock/404.django.html'
        ]

    def post(self, request, *args, **kwargs):
        """
        Sets ``request.POST`` data as a session variable, ``cradmin_uimock_postdata``,
        and redirects to the current full URL. In dispatch(), we pop this from session
        and adds it as the ``postdata`` template context data.

        This facilitates mocking simple form flows.
        """
        self.request.session['cradmin_uimock_postdata'] = self.request.POST.dict()
        return redirect(self.request.get_full_path())

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['postdata'] = self.session_postdata
        context['mockname'] = self.mockname
        return context



from builtins import object
from functools import update_wrapper

from django.urls import re_path

from .decorators import cradminview, has_access_to_cradmin_instance

#: The name of the app index view (the landing page for the app).
#: We do not enforce this, but we assume that each app has a
#: view with this name.
INDEXVIEW_NAME = 'INDEX'


class Url(object):
    """
    Url is mostly the same as func:`django.conf.urls.url`.
    You use Url to add urls to an app.
    """
    def __init__(self, regex, view, kwargs=None, name=None):
        """
        Args:
            regex: The URL regex.
            view: The view (E.g.: ``MyView.as_view()``).
            kwargs: Keyword arguments for the view.
            name: The name of the view. This just have to be unique within
                the :class:`.App` - the actual URL name is generated
                based on the app name and the :obj:`django_cradmin.crinstance.BaseCrAdminInstance.id`.
        """
        self.regex = regex
        self.view = view
        self.kwargs = kwargs
        self.name = name


class App(object):
    """
    A cradmin App.

    Added to a :class:`django_cradmin.crinstance.BaseCrAdminInstance`
    with :obj:`django_cradmin.crinstance.BaseCrAdminInstance.apps`.

    Attributes:
        appname (str): The name of the app.
        request (django.http.HttpRequest): Django request object for the current request.
        active_viewname (str): The name of the view we are currently rendering.
    """
    #: See :meth:`~.App.get_appurls`.
    appurls = []

    def __init__(self, appname, request, active_viewname):
        self.appname = appname
        self.request = request
        self.active_viewname = active_viewname

    def active_viewname_is_indexview(self):
        """
        Is the active viewname the app index view?

        Returns:
            bool: ``True`` if
        """
        return self.active_viewname == INDEXVIEW_NAME

    def is_crinstance_rolefrontpage_app(self):
        """
        Is this the :obj:`django_cradmin.crinstance.BaseCrAdminInstance.rolefrontpage_appname`

        Returns:
            bool: ``True`` if the appname of this app is the same as
            ``request.cradmin_instance.rolefrontpage_appname``.
        """
        return self.appname == self.request.cradmin_instance.rolefrontpage_appname

    def reverse_appurl(self, viewname, args=None, kwargs=None):
        """
        Works just like :func:`django.urls.reverse`, except
        that the name is the name given in :obj:`.appurls`, not the full
        name of the URL.

        This means that you should use this to reverse urls within this app.
        """
        return self.request.cradmin_instance.reverse_url(
            appname=self.appname, viewname=viewname,
            args=args, kwargs=kwargs)

    def reverse_appindexurl(self, args=None, kwargs=None):
        """
        Shortcut for::

            reverse_appurl(crapp.INDEXVIEW_NAME, args=args, kwargs=kwargs)
        """
        return self.reverse_appurl(viewname=INDEXVIEW_NAME, args=args, kwargs=kwargs)

    @classmethod
    def get_appurls(cls):
        """
        Get app URLs. Defaults to :obj:`~.App.appurls`.

        Returns:
            A list of :class:`.Url` objects.
        """
        return cls.appurls

    @classmethod
    def _wrap_view(cls, cradmin_instance_id, appname, view, viewname):
        def viewwrapper(request, *args, **kwargs):
            request.cradmin_app = cls(appname, request, active_viewname=viewname)
            return has_access_to_cradmin_instance(cradmin_instance_id, cradminview(view))(
                request, *args, **kwargs)

        # Take name and docstring from view
        update_wrapper(viewwrapper, view, updated=())

        return viewwrapper

    @classmethod
    def build_urls(cls, cradmin_instance_id, appname):
        """
        Used internally by :meth:`django_cradmin.crinstance.BaseCrAdminInstance.urls`
        to build urls for all views in the app.
        """
        urls = []
        for pattern in cls.get_appurls():
            urls.append(
                re_path(
                    pattern.regex, cls._wrap_view(cradmin_instance_id,
                                                  appname, pattern.view, pattern.name),
                    name='{}-{}-{}'.format(cradmin_instance_id, appname, pattern.name),
                    kwargs=pattern.kwargs))
        return urls

    def add_breadcrumb_list_items(self, breadcrumb_item_list):
        """
        Add items to the breadcrumb item list.

        If you completely override the :meth:`.get_breadcrumb_item_list_renderable` method
        without calling super (or calling this method explicitly), this method will have no effect.

        Examples::

            Simple example::

                def add_breadcrumb_list_items(self, breadcrumb_item_list):
                    breadcrumb_item_list.append(url='#', label='Test')


        Args:
            breadcrumb_item_list (django_cradmin.crbreadcrumb.BreadcrumbItemList): The breadcrumb item list
                to add items to.

        .. seealso:: :doc:`/crbreadcrumb`
        """

    def get_breadcrumb_item_list_renderable(self):
        """
        Get a breadcrumb item list renderable common for all views within this app.

        By default, this just uses ``request.cradmin_instance.get_breadcrumb_item_list_renderable()``
        (see :meth:`django_cradmin.crinstance.BaseCrAdminInstance.get_breadcrumb_item_list_renderable`).

        You will normally only want to override this if you want to customize how breadcrumbs
        are rendered for the views in this app. If you just need to add items to the breadcrumb item
        list, override :meth:`.add_breadcrumb_list_items`.

        If you override this, remember that the breadcrumb item list from
        ``request.cradmin_instance.get_breadcrumb_item_list_renderable()``
        can be ``None``, so if you use that method you have to remember to handle this.


        Returns:
            django_cradmin.crbreadcrumb.BreadcrumbItemList: A breadcrumb item list renderable object
                or ``None``.

        .. seealso:: :doc:`/crbreadcrumb`
        """
        breadcrumb_item_list = self.request.cradmin_instance.get_breadcrumb_item_list_renderable()
        if breadcrumb_item_list is not None:
            self.add_breadcrumb_list_items(breadcrumb_item_list=breadcrumb_item_list)
        return breadcrumb_item_list

    @property
    def page_cover_bem_block(self):
        """
        Override the default page cover BEM block for all views in the app.

        Used by :func:`django_cradmin.templatetags.cradmin_tags.cradmin_page_cover_bem_block` for more details.

        See also: :meth:`django_cradmin.crinstance.BaseCrAdminInstance.page_cover_bem_block`.
        """
        return self.request.cradmin_instance.page_cover_bem_block



# -*- coding: utf-8 -*-
# Generated by Django 1.11.5 on 2018-03-06 03:47
from django.db import migrations, models
import django_cradmin.deprecated_apps.cradmin_imagearchive.models


class Migration(migrations.Migration):

    dependencies = [
        ('cradmin_imagearchive', '0003_auto_20151017_0139'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archiveimage',
            name='image',
            field=models.FileField(help_text='Select an image to add to the archive.', max_length=255, null=True, upload_to=django_cradmin.deprecated_apps.cradmin_imagearchive.models.archiveimage_upload_to, verbose_name='image'),
        ),
    ]



import logging

from django import template
from django.urls import reverse

register = template.Library()
log = logging.getLogger(__name__)


@register.simple_tag(takes_context=True)
def cradmin_uimock_url(context, mockname=None, viewname=None):
    """
    Reverse the URL of a :class:`django_cradmin.viewhelpers.uimock.UiMock` view.

    Args:
        mockname (str): Name of the mock to link to (the name of the template without ``.django.html``).
            If this is not specified, we link to ``index.django.html``.
        viewname: The name of the mock view (the name specified for the view in urls.py).
            You do not need to specify this to link to mocks within the same view.
    """
    request = context['request']
    if viewname:
        path = reverse(viewname)
    else:
        path = request.path
        if not path.endswith('/'):
            path = path.rsplit('/', 1)[0] + '/'
    if mockname:
        path = '{}{}'.format(path, mockname)
    return path



import functools
import warnings


class CradminDeprecated(object):
    """
    Mark functions, classes, methods or modules as deprecated.

    Examples::

        Function::

            @CradminDeprecated()
            def myfunc():
                pass

        Function with message::

            @CradminDeprecated(message='This is deprecated and should not be used!')
            def myfunc():
                pass


        Method::

            class MyClass:

                @CradminDeprecated(message='This is deprecated and should not be used!')
                def mymethod(self):
                    pass

        Property::

            class MyClass:

                @property
                @CradminDeprecated(message='This is deprecated and should not be used!')
                def myproperty(self):
                    return 10

        Class::

            @CradminDeprecated(message='This is deprecated and should not be used!')
            class MyClass:
                pass

        Module::
            CradminDeprecated(message='This is deprecated and should not be used!')\
                .show_warning(name=__name__)
    """
    def __init__(self, message='Deprecated'):
        self.message = message

    def generate_message(self):
        return self.message

    def show_warning(self, name):
        with warnings.catch_warnings(record=False):
            warnings.simplefilter('always', DeprecationWarning)  # turn off filter
            warnings.warn(
                "{name}: {message}.".format(
                    name=name,
                    message=self.generate_message()),
                category=DeprecationWarning,
                stacklevel=2)

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):

            self.show_warning(name=func.__name__)
            return func(*args, **kwargs)
        return wrapper


class CradminDeprecatedSinceV4(CradminDeprecated):
    def __init__(self, message=''):
        super(CradminDeprecatedSinceV4, self).__init__(message=message)

    def generate_message(self):
        full_message = 'Deprecated in django_cradmin 4.x. Will be removed in a future release'
        if self.message:
            full_message = '{}: {}'.format(full_message, self.message)
        return full_message



from django import forms
from django import http
from django.contrib import messages
from django.contrib.contenttypes.fields import GenericForeignKey
from django.core import serializers

from . import formviewmixin
from . import previewmixin


class CreateUpdateViewMixin(previewmixin.PreviewMixin,
                            formviewmixin.FormViewMixin):
    """
    Mixin class for Update and Create views.

    .. note:: You should import this class with ``from django_cradmin import viewhelpers``,
        and refer to it using ``viewhelpers.formview.CreateUpdateViewMixin``.
    """

    #: The model class.
    model = None

    #: List of field names.
    #: See :meth:`.get_model_fields` and :meth:`.get_field_layout`.
    fields = None

    #: The field that should always be set to the current role.
    #: Removes the field from the form (see :meth:`.get_form`),
    #: and instead sets the value directly on the object in
    #: :meth:`.save_object`.
    roleid_field = None

    def get_model_class(self):
        """
        Get the model class.

        Defaults to :obj:`~.CreateUpdateViewMixin.model`.
        """
        return self.model

    def get_model_fields(self):
        """
        Get model fields. Defaults to :obj:`.fields`.
        """
        if self.fields:
            return self.fields
        else:
            fields = list(forms.fields_for_model(self.get_model_class(), fields=self.fields).keys())
            if self.roleid_field and self.roleid_field in fields:
                fields.remove(self.roleid_field)
            return fields

    def get_form(self, form_class=None):
        """
        If you set :obj:`.roleid_field`, we will remove that field from
        the form.

        .. note::
            The :obj:`.roleid_field` handling also works for GenericForeignKey
            fields (removes the content type and object pk field from the form).
        """
        form = super(CreateUpdateViewMixin, self).get_form(form_class)
        if self.roleid_field:
            roleid_fieldobject = getattr(self.get_model_class(), self.roleid_field, None)
            if isinstance(roleid_fieldobject, GenericForeignKey):
                for fieldname in roleid_fieldobject.fk_field, roleid_fieldobject.ct_field:
                    if fieldname in form.fields:
                        del form.fields[fieldname]
            else:
                if self.roleid_field in form.fields:
                    del form.fields[self.roleid_field]
        return form

    @property
    def model_verbose_name(self):
        """
        Get the verbose name of the model.
        """
        return self.get_model_class()._meta.verbose_name

    def add_create_update_view_mixin_context_data(self, context):
        """
        Must be called in :meth:`.get_context_data` in subclasses
        to add the required context data.

        Args:
            context: The template context data.
        """
        self.add_preview_mixin_context_data(context=context)
        self.add_formview_mixin_context_data(context=context)
        context['model_verbose_name'] = self.model_verbose_name

    def set_automatic_attributes(self, obj, form):
        """
        Called by :meth:`.save_object` to set automatic attributes for the
        object before it is saved.

        This is where we handle :obj:`.roleid_field`, but you can override this
        to set your own automatic attributes. Just remember to call ``super``
        if you want to keep the :obj:`.roleid_field` magic.

        Parameters:
            obj: The object you are about to save.
            form: The cleaned form.
        """
        if self.roleid_field:
            setattr(obj, self.roleid_field, self.request.cradmin_role)

    def save_object(self, form, commit=True):
        """
        Save the object. You can override this to customize how the
        form is turned into a saved object.

        Make sure you call ``super`` if you override this (see the docs for the commit parameter).
        If you do not, you will loose the automatic handling of obj:`.roleid_field`.

        Parameters:
            commit (boolean): If this is ``False``, the object is returned
                unsaved. Very useful when you want to manipulate the object
                before saving it in a subclass.

        Returns:
            The saved object.
        """
        obj = form.save(commit=False)
        self.set_automatic_attributes(obj, form)
        if commit:
            obj = form.save(commit=True)
        return obj

    def form_valid(self, form):
        """
        If the form is valid, save the associated model.
        """
        if self.preview_requested():
            self.store_preview_in_session(self.serialize_preview(form))
            return self.render_to_response(self.get_context_data(form=form, show_preview=True))
        else:
            self.object = self.save_object(form)
            self.form_saved(self.object)
            self.add_success_messages(self.object)
            return http.HttpResponseRedirect(self.get_success_url())

    def form_saved(self, object):
        """
        Called after the form has been successfully saved.
        The ``object`` is the saved object.

        Does nothing by default, but you can override it if you need to
        do something extra post save.
        """
        pass

    def get_success_message(self, obj):
        """
        Override this to provide a success message.

        The ``obj`` is the saved object.

        Used by :meth:`.add_success_messages`.
        """
        return None

    def add_success_messages(self, obj):
        """
        Called after the form has been saved, and after :meth:`.form_saved` has been called.

        The ``obj`` is the saved obj.

        Defaults to add :meth:`.get_success_message` as a django messages
        success message if :meth:`.get_success_message` returns anything.

        You can override this to add multiple messages or to show messages in some other way.
        """
        success_message = self.get_success_message(obj)
        if success_message:
            messages.success(self.request, success_message)

    def serialize_preview(self, form):
        """
        Seralize for preview.

        Defaults to serializing the object as JSON using ``django.core.serializers``.
        You can safely override this, but you will also have to override
        :meth:`deserialize_preview`.
        """
        return serializers.serialize('json', [self.save_object(form, commit=False)])

    @classmethod
    def deserialize_preview(cls, serialized):
        """
        Deseralize a preview serialized with :meth:`.serialize_preview`.

        You must override this and :meth:`.serialize_preview` - they work together
        to send the preview to the preview View.
        """
        return list(serializers.deserialize('json', serialized))[0].object

    @classmethod
    def get_preview_sessionkey(cls):
        """
        Get the session key used for preview. You should not
        need to override this.

        Unlike the default implementation of this method from
        :class:`django_cradmin.viewhelpers.formbase.PreviewMixin`,
        we use the model class module and name as the session key.
        This is simply because we do not want it to matter if
        you fetch preview data from create or update views
        for the same model (to simplify implementing preview views).
        """
        sessionkey = 'django_cradmin__{module}.{classname}'.format(
            module=cls.model.__module__,
            classname=cls.model.__name__)
        return sessionkey



from django.urls import path
from django_cradmin.apps.cradmin_register_account.views import register_account

urlpatterns = [
    path('',
         register_account.RegisterAccountView.as_view(),
         name="cradmin-register-account"),
]



from django.urls import path, re_path

from django_cradmin.apps.cradmin_kss_styleguide.views import exampleview
from .views import styleguideview

urlpatterns = [
    path('', styleguideview.GuideListView.as_view(),
         name='cradmin_kss_styleguide_guides'),
    re_path(r'^guide/(?P<unique_id>[a-zA-Z_.0-9-]+)/(?P<prefix>[^/]+)?',
            styleguideview.GuideView.as_view(),
            name='cradmin_kss_styleguide_guide'),
    re_path(r'^example/(?P<unique_id>[a-zA-Z_.0-9-]+)/(?P<section>[^/]+)/(?P<exampleindex>\d+)',
            exampleview.ExampleView.as_view(),
            name='cradmin_kss_styleguide_example'),
]
